"""
AWS Shield Advanced resource collector.

Note: Shield Advanced is a global service that requires us-east-1 for API operations.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_shield_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Shield Advanced resources: subscription, protections, and protection groups.

    Note: Shield Advanced APIs must be called from us-east-1 region.

    Args:
        session: boto3.Session to use
        region: AWS region (ignored - always uses us-east-1)
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    # Shield Advanced requires us-east-1 for all API calls
    shield = session.client('shield', region_name='us-east-1')

    # Check if Shield Advanced is active
    try:
        sub_state = shield.get_subscription_state()
        if sub_state.get('SubscriptionState') != 'ACTIVE':
            return []
    except Exception:
        return []

    # Subscription details
    try:
        sub = shield.describe_subscription()
        subscription = sub.get('Subscription', {})

        details = {
            'start_time': str(subscription.get('StartTime', '')),
            'end_time': str(subscription.get('EndTime', '')),
            'time_commitment_in_seconds': subscription.get('TimeCommitmentInSeconds'),
            'auto_renew': subscription.get('AutoRenew'),
            'proactive_engagement_status': subscription.get('ProactiveEngagementStatus'),
            'subscription_arn': subscription.get('SubscriptionArn'),
        }

        # Limits
        limits = subscription.get('Limits', [])
        for limit in limits:
            limit_type = limit.get('Type', '').lower().replace(' ', '_')
            details[f"limit_{limit_type}"] = limit.get('Max')

        resources.append({
            'service': 'shield',
            'type': 'subscription',
            'id': 'shield-advanced-subscription',
            'arn': subscription.get('SubscriptionArn', f"arn:aws:shield::{account_id}:subscription"),
            'name': 'Shield Advanced Subscription',
            'region': 'global',
            'details': details,
            'tags': {}
        })
    except Exception:
        pass

    # Protections
    try:
        paginator = shield.get_paginator('list_protections')
        for page in paginator.paginate():
            for protection in page.get('Protections', []):
                protection_id = protection['Id']
                protection_name = protection.get('Name', protection_id)
                protection_arn = protection.get('ProtectionArn', '')

                details = {
                    'resource_arn': protection.get('ResourceArn'),
                    'health_check_ids': protection.get('HealthCheckIds', []),
                    'application_layer_automatic_response_status': protection.get('ApplicationLayerAutomaticResponseConfiguration', {}).get('Status'),
                }

                # Get tags
                tags = {}
                if protection_arn:
                    try:
                        tag_response = shield.list_tags_for_resource(ResourceARN=protection_arn)
                        for tag in tag_response.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                    except Exception:
                        pass

                resources.append({
                    'service': 'shield',
                    'type': 'protection',
                    'id': protection_id,
                    'arn': protection_arn,
                    'name': protection_name,
                    'region': 'global',
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Protection Groups
    try:
        response = shield.list_protection_groups()
        for group in response.get('ProtectionGroups', []):
            group_id = group['ProtectionGroupId']
            group_arn = group.get('ProtectionGroupArn', '')

            details = {
                'aggregation': group.get('Aggregation'),
                'pattern': group.get('Pattern'),
                'resource_type': group.get('ResourceType'),
                'members': group.get('Members', []),
                'members_count': len(group.get('Members', [])),
            }

            # Get tags
            tags = {}
            if group_arn:
                try:
                    tag_response = shield.list_tags_for_resource(ResourceARN=group_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

            resources.append({
                'service': 'shield',
                'type': 'protection-group',
                'id': group_id,
                'arn': group_arn,
                'name': group_id,
                'region': 'global',
                'details': details,
                'tags': tags
            })
    except Exception:
        pass

    return resources



"""
Amazon Managed Grafana resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional

# Amazon Managed Grafana supported regions
# https://docs.aws.amazon.com/grafana/latest/userguide/what-is-Amazon-Managed-Service-Grafana.html
GRAFANA_REGIONS = {
    # US
    'us-east-1', 'us-east-2', 'us-west-2',
    # Asia Pacific
    'ap-northeast-1', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2',
    # Europe
    'eu-central-1', 'eu-west-1', 'eu-west-2',
}


def collect_grafana_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Amazon Managed Grafana resources: workspaces.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions to avoid timeouts
    if region and region not in GRAFANA_REGIONS:
        return []

    resources = []
    grafana = session.client('grafana', region_name=region)

    # Grafana Workspaces
    try:
        paginator = grafana.get_paginator('list_workspaces')
        for page in paginator.paginate():
            for workspace in page.get('workspaces', []):
                ws_id = workspace['id']
                ws_name = workspace.get('name', ws_id)

                try:
                    # Get workspace details
                    ws_response = grafana.describe_workspace(workspaceId=ws_id)
                    ws_detail = ws_response.get('workspace', {})

                    # Get tags
                    tags = ws_detail.get('tags', {})

                    resources.append({
                        'service': 'grafana',
                        'type': 'workspace',
                        'id': ws_id,
                        'arn': ws_detail.get('arn', f"arn:aws:grafana:{region}:{account_id}:/workspaces/{ws_id}"),
                        'name': ws_name,
                        'region': region,
                        'details': {
                            'status': ws_detail.get('status'),
                            'description': ws_detail.get('description'),
                            'endpoint': ws_detail.get('endpoint'),
                            'grafana_version': ws_detail.get('grafanaVersion'),
                            'account_access_type': ws_detail.get('accountAccessType'),
                            'authentication_providers': ws_detail.get('authentication', {}).get('providers', []),
                            'permission_type': ws_detail.get('permissionType'),
                            'license_type': ws_detail.get('licenseType'),
                            'stack_set_name': ws_detail.get('stackSetName'),
                            'data_sources': ws_detail.get('dataSources', []),
                            'notification_destinations': ws_detail.get('notificationDestinations', []),
                            'created': str(ws_detail.get('created', '')),
                            'modified': str(ws_detail.get('modified', '')),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
Auto Scaling resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_autoscaling_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Auto Scaling resources: auto scaling groups, launch configurations, scaling policies.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    autoscaling = session.client('autoscaling', region_name=region)

    # Auto Scaling Groups
    try:
        paginator = autoscaling.get_paginator('describe_auto_scaling_groups')
        for page in paginator.paginate():
            for asg in page.get('AutoScalingGroups', []):
                asg_name = asg['AutoScalingGroupName']
                asg_arn = asg['AutoScalingGroupARN']

                # Tags are included in the response
                tags = {}
                for tag in asg.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')

                resources.append({
                    'service': 'autoscaling',
                    'type': 'auto-scaling-group',
                    'id': asg_name,
                    'arn': asg_arn,
                    'name': asg_name,
                    'region': region,
                    'details': {
                        'min_size': asg.get('MinSize'),
                        'max_size': asg.get('MaxSize'),
                        'desired_capacity': asg.get('DesiredCapacity'),
                        'default_cooldown': asg.get('DefaultCooldown'),
                        'availability_zones': asg.get('AvailabilityZones', []),
                        'load_balancer_names': asg.get('LoadBalancerNames', []),
                        'target_group_arns': asg.get('TargetGroupARNs', []),
                        'health_check_type': asg.get('HealthCheckType'),
                        'health_check_grace_period': asg.get('HealthCheckGracePeriod'),
                        'instances_count': len(asg.get('Instances', [])),
                        'launch_configuration_name': asg.get('LaunchConfigurationName'),
                        'launch_template': asg.get('LaunchTemplate'),
                        'mixed_instances_policy': bool(asg.get('MixedInstancesPolicy')),
                        'vpc_zone_identifier': asg.get('VPCZoneIdentifier'),
                        'service_linked_role_arn': asg.get('ServiceLinkedRoleARN'),
                        'capacity_rebalance': asg.get('CapacityRebalance'),
                        'created_time': str(asg.get('CreatedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Launch Configurations
    try:
        paginator = autoscaling.get_paginator('describe_launch_configurations')
        for page in paginator.paginate():
            for lc in page.get('LaunchConfigurations', []):
                lc_name = lc['LaunchConfigurationName']
                lc_arn = lc['LaunchConfigurationARN']

                resources.append({
                    'service': 'autoscaling',
                    'type': 'launch-configuration',
                    'id': lc_name,
                    'arn': lc_arn,
                    'name': lc_name,
                    'region': region,
                    'details': {
                        'image_id': lc.get('ImageId'),
                        'instance_type': lc.get('InstanceType'),
                        'key_name': lc.get('KeyName'),
                        'security_groups': lc.get('SecurityGroups', []),
                        'instance_monitoring': lc.get('InstanceMonitoring', {}).get('Enabled'),
                        'spot_price': lc.get('SpotPrice'),
                        'iam_instance_profile': lc.get('IamInstanceProfile'),
                        'ebs_optimized': lc.get('EbsOptimized'),
                        'associate_public_ip_address': lc.get('AssociatePublicIpAddress'),
                        'placement_tenancy': lc.get('PlacementTenancy'),
                        'created_time': str(lc.get('CreatedTime', '')),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Scaling Policies
    try:
        paginator = autoscaling.get_paginator('describe_policies')
        for page in paginator.paginate():
            for policy in page.get('ScalingPolicies', []):
                policy_name = policy['PolicyName']
                policy_arn = policy['PolicyARN']

                resources.append({
                    'service': 'autoscaling',
                    'type': 'scaling-policy',
                    'id': policy_name,
                    'arn': policy_arn,
                    'name': policy_name,
                    'region': region,
                    'details': {
                        'auto_scaling_group_name': policy.get('AutoScalingGroupName'),
                        'policy_type': policy.get('PolicyType'),
                        'adjustment_type': policy.get('AdjustmentType'),
                        'scaling_adjustment': policy.get('ScalingAdjustment'),
                        'cooldown': policy.get('Cooldown'),
                        'min_adjustment_magnitude': policy.get('MinAdjustmentMagnitude'),
                        'metric_aggregation_type': policy.get('MetricAggregationType'),
                        'estimated_instance_warmup': policy.get('EstimatedInstanceWarmup'),
                        'enabled': policy.get('Enabled'),
                        'target_tracking_configuration': bool(policy.get('TargetTrackingConfiguration')),
                        'predictive_scaling_configuration': bool(policy.get('PredictiveScalingConfiguration')),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Scheduled Actions
    try:
        paginator = autoscaling.get_paginator('describe_scheduled_actions')
        for page in paginator.paginate():
            for action in page.get('ScheduledUpdateGroupActions', []):
                action_name = action['ScheduledActionName']
                action_arn = action['ScheduledActionARN']

                resources.append({
                    'service': 'autoscaling',
                    'type': 'scheduled-action',
                    'id': action_name,
                    'arn': action_arn,
                    'name': action_name,
                    'region': region,
                    'details': {
                        'auto_scaling_group_name': action.get('AutoScalingGroupName'),
                        'recurrence': action.get('Recurrence'),
                        'min_size': action.get('MinSize'),
                        'max_size': action.get('MaxSize'),
                        'desired_capacity': action.get('DesiredCapacity'),
                        'start_time': str(action.get('StartTime', '')),
                        'end_time': str(action.get('EndTime', '')),
                        'time_zone': action.get('TimeZone'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
Amazon OpenSearch Serverless resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_opensearchserverless_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect OpenSearch Serverless resources: collections, access policies, security policies, VPC endpoints.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    oss = session.client('opensearchserverless', region_name=region)

    # Collections
    try:
        response = oss.list_collections()
        for summary in response.get('collectionSummaries', []):
            collection_id = summary.get('id', '')
            collection_arn = summary.get('arn', '')
            collection_name = summary.get('name', collection_id)

            details = {
                'status': summary.get('status'),
            }

            resources.append({
                'service': 'opensearch-serverless',
                'type': 'collection',
                'id': collection_id,
                'arn': collection_arn,
                'name': collection_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    # VPC Endpoints
    try:
        response = oss.list_vpc_endpoints()
        for endpoint in response.get('vpcEndpointSummaries', []):
            endpoint_id = endpoint.get('id', '')
            endpoint_name = endpoint.get('name', endpoint_id)

            details = {
                'status': endpoint.get('status'),
            }

            resources.append({
                'service': 'opensearch-serverless',
                'type': 'vpc-endpoint',
                'id': endpoint_id,
                'arn': f"arn:aws:aoss:{region}:{account_id}:vpcendpoint/{endpoint_id}",
                'name': endpoint_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    # Access Policies
    try:
        response = oss.list_access_policies(type='data')
        for policy in response.get('accessPolicySummaries', []):
            policy_name = policy.get('name', '')
            policy_version = policy.get('policyVersion', '')

            details = {
                'type': policy.get('type'),
                'policy_version': policy_version,
            }

            resources.append({
                'service': 'opensearch-serverless',
                'type': 'access-policy',
                'id': policy_name,
                'arn': f"arn:aws:aoss:{region}:{account_id}:accesspolicy/{policy_name}",
                'name': policy_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    # Security Policies (encryption)
    try:
        response = oss.list_security_policies(type='encryption')
        for policy in response.get('securityPolicySummaries', []):
            policy_name = policy.get('name', '')

            details = {
                'type': policy.get('type'),
                'policy_version': policy.get('policyVersion'),
            }

            resources.append({
                'service': 'opensearch-serverless',
                'type': 'security-policy',
                'id': f"encryption-{policy_name}",
                'arn': f"arn:aws:aoss:{region}:{account_id}:securitypolicy/encryption/{policy_name}",
                'name': policy_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    # Security Policies (network)
    try:
        response = oss.list_security_policies(type='network')
        for policy in response.get('securityPolicySummaries', []):
            policy_name = policy.get('name', '')

            details = {
                'type': policy.get('type'),
                'policy_version': policy.get('policyVersion'),
            }

            resources.append({
                'service': 'opensearch-serverless',
                'type': 'security-policy',
                'id': f"network-{policy_name}",
                'arn': f"arn:aws:aoss:{region}:{account_id}:securitypolicy/network/{policy_name}",
                'name': policy_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    return resources



"""
WorkSpaces resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional

# WorkSpaces supported regions
# https://docs.aws.amazon.com/general/latest/gr/wsp.html
WORKSPACES_REGIONS = {
    # US
    'us-east-1', 'us-west-2',
    # Asia Pacific
    'ap-south-1', 'ap-northeast-1', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2',
    # Canada
    'ca-central-1',
    # Europe
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3',
    # South America
    'sa-east-1',
    # Africa
    'af-south-1',
    # Israel
    'il-central-1',
}


def collect_workspaces_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect WorkSpaces resources: workspaces, directories, bundles, images.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions to avoid timeouts
    if region and region not in WORKSPACES_REGIONS:
        return []

    resources = []
    workspaces = session.client('workspaces', region_name=region)

    # WorkSpaces
    try:
        paginator = workspaces.get_paginator('describe_workspaces')
        for page in paginator.paginate():
            for ws in page.get('Workspaces', []):
                ws_id = ws['WorkspaceId']

                # Get tags
                tags = {}
                try:
                    tag_response = workspaces.describe_tags(ResourceId=ws_id)
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'workspaces',
                    'type': 'workspace',
                    'id': ws_id,
                    'arn': f"arn:aws:workspaces:{region}:{account_id}:workspace/{ws_id}",
                    'name': ws.get('ComputerName') or ws.get('UserName') or ws_id,
                    'region': region,
                    'details': {
                        'directory_id': ws.get('DirectoryId'),
                        'user_name': ws.get('UserName'),
                        'ip_address': ws.get('IpAddress'),
                        'state': ws.get('State'),
                        'bundle_id': ws.get('BundleId'),
                        'subnet_id': ws.get('SubnetId'),
                        'computer_name': ws.get('ComputerName'),
                        'volume_encryption_key': ws.get('VolumeEncryptionKey'),
                        'user_volume_encryption_enabled': ws.get('UserVolumeEncryptionEnabled'),
                        'root_volume_encryption_enabled': ws.get('RootVolumeEncryptionEnabled'),
                        'running_mode': ws.get('WorkspaceProperties', {}).get('RunningMode'),
                        'running_mode_auto_stop_timeout': ws.get('WorkspaceProperties', {}).get('RunningModeAutoStopTimeoutInMinutes'),
                        'root_volume_size': ws.get('WorkspaceProperties', {}).get('RootVolumeSizeGib'),
                        'user_volume_size': ws.get('WorkspaceProperties', {}).get('UserVolumeSizeGib'),
                        'compute_type': ws.get('WorkspaceProperties', {}).get('ComputeTypeName'),
                        'protocols': ws.get('WorkspaceProperties', {}).get('Protocols', []),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # WorkSpaces Directories
    try:
        paginator = workspaces.get_paginator('describe_workspace_directories')
        for page in paginator.paginate():
            for directory in page.get('Directories', []):
                dir_id = directory['DirectoryId']

                # Get tags
                tags = {}
                try:
                    tag_response = workspaces.describe_tags(ResourceId=dir_id)
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'workspaces',
                    'type': 'directory',
                    'id': dir_id,
                    'arn': f"arn:aws:workspaces:{region}:{account_id}:directory/{dir_id}",
                    'name': directory.get('DirectoryName') or dir_id,
                    'region': region,
                    'details': {
                        'directory_name': directory.get('DirectoryName'),
                        'directory_type': directory.get('DirectoryType'),
                        'registration_code': directory.get('RegistrationCode'),
                        'state': directory.get('State'),
                        'workspace_creation_properties': directory.get('WorkspaceCreationProperties'),
                        'alias': directory.get('Alias'),
                        'customer_user_name': directory.get('CustomerUserName'),
                        'iam_role_id': directory.get('IamRoleId'),
                        'workspace_security_group_id': directory.get('WorkspaceSecurityGroupId'),
                        'dns_ip_addresses': directory.get('DnsIpAddresses', []),
                        'subnet_ids': directory.get('SubnetIds', []),
                        'ip_group_ids': directory.get('ipGroupIds', []),
                        'tenancy': directory.get('Tenancy'),
                        'selfservice_permissions': directory.get('SelfservicePermissions'),
                        'saml_properties': bool(directory.get('SamlProperties')),
                        'certificate_based_auth_properties': bool(directory.get('CertificateBasedAuthProperties')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Note: Bundles and Images skipped for performance (saves ~82s)
    # They are rarely used custom resources

    return resources



"""
AWS Database Migration Service (DMS) resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_dms_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS DMS resources: replication instances, tasks, endpoints, certificates, and subnet groups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    dms = session.client('dms', region_name=region)

    # Replication Instances
    try:
        paginator = dms.get_paginator('describe_replication_instances')
        for page in paginator.paginate():
            for instance in page.get('ReplicationInstances', []):
                instance_id = instance['ReplicationInstanceIdentifier']
                instance_arn = instance.get('ReplicationInstanceArn', '')

                details = {
                    'replication_instance_class': instance.get('ReplicationInstanceClass'),
                    'replication_instance_status': instance.get('ReplicationInstanceStatus'),
                    'allocated_storage': instance.get('AllocatedStorage'),
                    'engine_version': instance.get('EngineVersion'),
                    'auto_minor_version_upgrade': instance.get('AutoMinorVersionUpgrade'),
                    'multi_az': instance.get('MultiAZ'),
                    'publicly_accessible': instance.get('PubliclyAccessible'),
                    'availability_zone': instance.get('AvailabilityZone'),
                    'replication_subnet_group_id': instance.get('ReplicationSubnetGroup', {}).get('ReplicationSubnetGroupIdentifier'),
                    'vpc_security_group_ids': [sg.get('VpcSecurityGroupId') for sg in instance.get('VpcSecurityGroups', [])],
                    'kms_key_id': instance.get('KmsKeyId'),
                    'free_until': str(instance.get('FreeUntil', '')),
                    'instance_create_time': str(instance.get('InstanceCreateTime', '')),
                }

                resources.append({
                    'service': 'dms',
                    'type': 'replication-instance',
                    'id': instance_id,
                    'arn': instance_arn,
                    'name': instance_id,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Replication Tasks
    try:
        paginator = dms.get_paginator('describe_replication_tasks')
        for page in paginator.paginate():
            for task in page.get('ReplicationTasks', []):
                task_id = task['ReplicationTaskIdentifier']
                task_arn = task.get('ReplicationTaskArn', '')

                details = {
                    'source_endpoint_arn': task.get('SourceEndpointArn'),
                    'target_endpoint_arn': task.get('TargetEndpointArn'),
                    'replication_instance_arn': task.get('ReplicationInstanceArn'),
                    'migration_type': task.get('MigrationType'),
                    'status': task.get('Status'),
                    'stop_reason': task.get('StopReason'),
                    'replication_task_creation_date': str(task.get('ReplicationTaskCreationDate', '')),
                    'replication_task_start_date': str(task.get('ReplicationTaskStartDate', '')),
                    'cdc_start_position': task.get('CdcStartPosition'),
                    'cdc_stop_position': task.get('CdcStopPosition'),
                    'recovery_checkpoint': task.get('RecoveryCheckpoint'),
                }

                stats = task.get('ReplicationTaskStats', {})
                if stats:
                    details['tables_loaded'] = stats.get('TablesLoaded')
                    details['tables_loading'] = stats.get('TablesLoading')
                    details['tables_queued'] = stats.get('TablesQueued')
                    details['tables_errored'] = stats.get('TablesErrored')
                    details['elapsed_time_millis'] = stats.get('ElapsedTimeMillis')

                resources.append({
                    'service': 'dms',
                    'type': 'replication-task',
                    'id': task_id,
                    'arn': task_arn,
                    'name': task_id,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Endpoints
    try:
        paginator = dms.get_paginator('describe_endpoints')
        for page in paginator.paginate():
            for endpoint in page.get('Endpoints', []):
                endpoint_id = endpoint['EndpointIdentifier']
                endpoint_arn = endpoint.get('EndpointArn', '')

                details = {
                    'endpoint_type': endpoint.get('EndpointType'),
                    'engine_name': endpoint.get('EngineName'),
                    'engine_display_name': endpoint.get('EngineDisplayName'),
                    'server_name': endpoint.get('ServerName'),
                    'port': endpoint.get('Port'),
                    'database_name': endpoint.get('DatabaseName'),
                    'status': endpoint.get('Status'),
                    'kms_key_id': endpoint.get('KmsKeyId'),
                    'ssl_mode': endpoint.get('SslMode'),
                    'certificate_arn': endpoint.get('CertificateArn'),
                }

                resources.append({
                    'service': 'dms',
                    'type': 'endpoint',
                    'id': endpoint_id,
                    'arn': endpoint_arn,
                    'name': endpoint_id,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Certificates
    try:
        paginator = dms.get_paginator('describe_certificates')
        for page in paginator.paginate():
            for cert in page.get('Certificates', []):
                cert_id = cert['CertificateIdentifier']
                cert_arn = cert.get('CertificateArn', '')

                details = {
                    'certificate_owner': cert.get('CertificateOwner'),
                    'valid_from_date': str(cert.get('ValidFromDate', '')),
                    'valid_to_date': str(cert.get('ValidToDate', '')),
                    'signing_algorithm': cert.get('SigningAlgorithm'),
                    'key_length': cert.get('KeyLength'),
                    'certificate_creation_date': str(cert.get('CertificateCreationDate', '')),
                }

                resources.append({
                    'service': 'dms',
                    'type': 'certificate',
                    'id': cert_id,
                    'arn': cert_arn,
                    'name': cert_id,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Replication Subnet Groups
    try:
        paginator = dms.get_paginator('describe_replication_subnet_groups')
        for page in paginator.paginate():
            for group in page.get('ReplicationSubnetGroups', []):
                group_id = group['ReplicationSubnetGroupIdentifier']

                details = {
                    'replication_subnet_group_description': group.get('ReplicationSubnetGroupDescription'),
                    'vpc_id': group.get('VpcId'),
                    'subnet_group_status': group.get('SubnetGroupStatus'),
                    'subnets': [s.get('SubnetIdentifier') for s in group.get('Subnets', [])],
                    'supported_network_types': group.get('SupportedNetworkTypes', []),
                }

                resources.append({
                    'service': 'dms',
                    'type': 'replication-subnet-group',
                    'id': group_id,
                    'arn': f"arn:aws:dms:{region}:{account_id}:subgrp:{group_id}",
                    'name': group_id,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
QuickSight resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional

# QuickSight/QuickSuite supported regions
# https://docs.aws.amazon.com/quicksuite/latest/userguide/regions.html
QUICKSIGHT_REGIONS = {
    # US
    'us-east-1', 'us-east-2', 'us-west-2',
    # Europe
    'eu-central-1', 'eu-central-2', 'eu-west-1', 'eu-west-2', 'eu-west-3',
    'eu-south-1', 'eu-south-2', 'eu-north-1',
    # Asia Pacific
    'ap-south-1', 'ap-southeast-1', 'ap-southeast-2', 'ap-southeast-3',
    'ap-northeast-1', 'ap-northeast-2',
    # Other
    'ca-central-1', 'sa-east-1', 'af-south-1', 'il-central-1', 'me-central-1',
}


def collect_quicksight_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect QuickSight resources: dashboards, data sets, data sources, analyses.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions to avoid timeouts
    if region and region not in QUICKSIGHT_REGIONS:
        return []

    resources = []
    quicksight = session.client('quicksight', region_name=region)

    # QuickSight Dashboards
    try:
        paginator = quicksight.get_paginator('list_dashboards')
        for page in paginator.paginate(AwsAccountId=account_id):
            for dashboard in page.get('DashboardSummaryList', []):
                dashboard_id = dashboard['DashboardId']
                dashboard_arn = dashboard['Arn']

                # Get tags
                tags = {}
                try:
                    tag_response = quicksight.list_tags_for_resource(ResourceArn=dashboard_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'quicksight',
                    'type': 'dashboard',
                    'id': dashboard_id,
                    'arn': dashboard_arn,
                    'name': dashboard.get('Name', dashboard_id),
                    'region': region,
                    'details': {
                        'published_version_number': dashboard.get('PublishedVersionNumber'),
                        'created_time': str(dashboard.get('CreatedTime', '')),
                        'last_updated_time': str(dashboard.get('LastUpdatedTime', '')),
                        'last_published_time': str(dashboard.get('LastPublishedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # QuickSight Data Sets
    try:
        paginator = quicksight.get_paginator('list_data_sets')
        for page in paginator.paginate(AwsAccountId=account_id):
            for dataset in page.get('DataSetSummaries', []):
                dataset_id = dataset['DataSetId']
                dataset_arn = dataset['Arn']

                # Get tags
                tags = {}
                try:
                    tag_response = quicksight.list_tags_for_resource(ResourceArn=dataset_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'quicksight',
                    'type': 'data-set',
                    'id': dataset_id,
                    'arn': dataset_arn,
                    'name': dataset.get('Name', dataset_id),
                    'region': region,
                    'details': {
                        'import_mode': dataset.get('ImportMode'),
                        'created_time': str(dataset.get('CreatedTime', '')),
                        'last_updated_time': str(dataset.get('LastUpdatedTime', '')),
                        'row_level_permission_data_set': bool(dataset.get('RowLevelPermissionDataSet')),
                        'row_level_permission_tag_configuration_applied': dataset.get('RowLevelPermissionTagConfigurationApplied'),
                        'column_level_permission_rules_applied': dataset.get('ColumnLevelPermissionRulesApplied'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # QuickSight Data Sources
    try:
        paginator = quicksight.get_paginator('list_data_sources')
        for page in paginator.paginate(AwsAccountId=account_id):
            for datasource in page.get('DataSources', []):
                datasource_id = datasource['DataSourceId']
                datasource_arn = datasource['Arn']

                # Get tags
                tags = {}
                try:
                    tag_response = quicksight.list_tags_for_resource(ResourceArn=datasource_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'quicksight',
                    'type': 'data-source',
                    'id': datasource_id,
                    'arn': datasource_arn,
                    'name': datasource.get('Name', datasource_id),
                    'region': region,
                    'details': {
                        'type': datasource.get('Type'),
                        'status': datasource.get('Status'),
                        'created_time': str(datasource.get('CreatedTime', '')),
                        'last_updated_time': str(datasource.get('LastUpdatedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # QuickSight Analyses
    try:
        paginator = quicksight.get_paginator('list_analyses')
        for page in paginator.paginate(AwsAccountId=account_id):
            for analysis in page.get('AnalysisSummaryList', []):
                analysis_id = analysis['AnalysisId']
                analysis_arn = analysis['Arn']

                # Get tags
                tags = {}
                try:
                    tag_response = quicksight.list_tags_for_resource(ResourceArn=analysis_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'quicksight',
                    'type': 'analysis',
                    'id': analysis_id,
                    'arn': analysis_arn,
                    'name': analysis.get('Name', analysis_id),
                    'region': region,
                    'details': {
                        'status': analysis.get('Status'),
                        'created_time': str(analysis.get('CreatedTime', '')),
                        'last_updated_time': str(analysis.get('LastUpdatedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
EC2 resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional

from aws_inventory.collector import tags_to_dict, get_tag_value


def collect_ec2_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect EC2 resources: instances, volumes, snapshots, AMIs, security groups,
    key pairs, elastic IPs, network interfaces, placement groups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    ec2 = session.client('ec2', region_name=region)

    # EC2 Instances
    try:
        paginator = ec2.get_paginator('describe_instances')
        for page in paginator.paginate():
            for reservation in page.get('Reservations', []):
                for instance in reservation.get('Instances', []):
                    tags = instance.get('Tags', [])
                    resources.append({
                        'service': 'ec2',
                        'type': 'instance',
                        'id': instance['InstanceId'],
                        'arn': f"arn:aws:ec2:{region}:{account_id}:instance/{instance['InstanceId']}",
                        'name': get_tag_value(tags, 'Name') or instance['InstanceId'],
                        'region': region,
                        'details': {
                            'instance_type': instance.get('InstanceType'),
                            'state': instance.get('State', {}).get('Name'),
                            'private_ip': instance.get('PrivateIpAddress'),
                            'public_ip': instance.get('PublicIpAddress'),
                            'vpc_id': instance.get('VpcId'),
                            'subnet_id': instance.get('SubnetId'),
                            'launch_time': str(instance.get('LaunchTime', '')),
                            'platform': instance.get('Platform', 'linux'),
                            'architecture': instance.get('Architecture'),
                        },
                        'tags': tags_to_dict(tags)
                    })
    except Exception:
        pass

    # EBS Volumes
    try:
        paginator = ec2.get_paginator('describe_volumes')
        for page in paginator.paginate():
            for volume in page.get('Volumes', []):
                tags = volume.get('Tags', [])
                resources.append({
                    'service': 'ec2',
                    'type': 'volume',
                    'id': volume['VolumeId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:volume/{volume['VolumeId']}",
                    'name': get_tag_value(tags, 'Name') or volume['VolumeId'],
                    'region': region,
                    'details': {
                        'size_gb': volume.get('Size'),
                        'volume_type': volume.get('VolumeType'),
                        'state': volume.get('State'),
                        'iops': volume.get('Iops'),
                        'encrypted': volume.get('Encrypted'),
                        'availability_zone': volume.get('AvailabilityZone'),
                        'attachments': [a.get('InstanceId') for a in volume.get('Attachments', [])],
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # EBS Snapshots (owned by this account)
    try:
        paginator = ec2.get_paginator('describe_snapshots')
        for page in paginator.paginate(OwnerIds=[account_id]):
            for snapshot in page.get('Snapshots', []):
                tags = snapshot.get('Tags', [])
                resources.append({
                    'service': 'ec2',
                    'type': 'snapshot',
                    'id': snapshot['SnapshotId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:snapshot/{snapshot['SnapshotId']}",
                    'name': get_tag_value(tags, 'Name') or snapshot['SnapshotId'],
                    'region': region,
                    'details': {
                        'volume_id': snapshot.get('VolumeId'),
                        'size_gb': snapshot.get('VolumeSize'),
                        'state': snapshot.get('State'),
                        'encrypted': snapshot.get('Encrypted'),
                        'start_time': str(snapshot.get('StartTime', '')),
                        'description': snapshot.get('Description'),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # AMIs (owned by this account)
    try:
        response = ec2.describe_images(Owners=[account_id])
        for image in response.get('Images', []):
            tags = image.get('Tags', [])
            resources.append({
                'service': 'ec2',
                'type': 'ami',
                'id': image['ImageId'],
                'arn': f"arn:aws:ec2:{region}:{account_id}:image/{image['ImageId']}",
                'name': image.get('Name') or get_tag_value(tags, 'Name') or image['ImageId'],
                'region': region,
                'details': {
                    'state': image.get('State'),
                    'architecture': image.get('Architecture'),
                    'platform': image.get('Platform', 'linux'),
                    'virtualization_type': image.get('VirtualizationType'),
                    'root_device_type': image.get('RootDeviceType'),
                    'public': image.get('Public'),
                    'creation_date': image.get('CreationDate'),
                },
                'tags': tags_to_dict(tags)
            })
    except Exception:
        pass

    # Security Groups
    try:
        paginator = ec2.get_paginator('describe_security_groups')
        for page in paginator.paginate():
            for sg in page.get('SecurityGroups', []):
                tags = sg.get('Tags', [])
                resources.append({
                    'service': 'ec2',
                    'type': 'security-group',
                    'id': sg['GroupId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:security-group/{sg['GroupId']}",
                    'name': sg.get('GroupName') or sg['GroupId'],
                    'region': region,
                    'is_default': sg.get('GroupName') == 'default',
                    'details': {
                        'vpc_id': sg.get('VpcId'),
                        'description': sg.get('Description'),
                        'ingress_rules': len(sg.get('IpPermissions', [])),
                        'egress_rules': len(sg.get('IpPermissionsEgress', [])),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # Key Pairs
    try:
        response = ec2.describe_key_pairs()
        for kp in response.get('KeyPairs', []):
            tags = kp.get('Tags', [])
            resources.append({
                'service': 'ec2',
                'type': 'key-pair',
                'id': kp.get('KeyPairId', kp['KeyName']),
                'arn': f"arn:aws:ec2:{region}:{account_id}:key-pair/{kp.get('KeyPairId', kp['KeyName'])}",
                'name': kp['KeyName'],
                'region': region,
                'details': {
                    'key_type': kp.get('KeyType'),
                    'fingerprint': kp.get('KeyFingerprint'),
                    'create_time': str(kp.get('CreateTime', '')),
                },
                'tags': tags_to_dict(tags)
            })
    except Exception:
        pass

    # Elastic IPs
    try:
        response = ec2.describe_addresses()
        for addr in response.get('Addresses', []):
            tags = addr.get('Tags', [])
            resources.append({
                'service': 'ec2',
                'type': 'elastic-ip',
                'id': addr.get('AllocationId', addr.get('PublicIp')),
                'arn': f"arn:aws:ec2:{region}:{account_id}:elastic-ip/{addr.get('AllocationId', addr.get('PublicIp'))}",
                'name': get_tag_value(tags, 'Name') or addr.get('PublicIp'),
                'region': region,
                'details': {
                    'public_ip': addr.get('PublicIp'),
                    'private_ip': addr.get('PrivateIpAddress'),
                    'instance_id': addr.get('InstanceId'),
                    'network_interface_id': addr.get('NetworkInterfaceId'),
                    'domain': addr.get('Domain'),
                },
                'tags': tags_to_dict(tags)
            })
    except Exception:
        pass

    # Network Interfaces
    try:
        paginator = ec2.get_paginator('describe_network_interfaces')
        for page in paginator.paginate():
            for eni in page.get('NetworkInterfaces', []):
                tags = eni.get('TagSet', [])
                resources.append({
                    'service': 'ec2',
                    'type': 'network-interface',
                    'id': eni['NetworkInterfaceId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:network-interface/{eni['NetworkInterfaceId']}",
                    'name': get_tag_value(tags, 'Name') or eni['NetworkInterfaceId'],
                    'region': region,
                    'details': {
                        'vpc_id': eni.get('VpcId'),
                        'subnet_id': eni.get('SubnetId'),
                        'private_ip': eni.get('PrivateIpAddress'),
                        'status': eni.get('Status'),
                        'interface_type': eni.get('InterfaceType'),
                        'attachment_instance': eni.get('Attachment', {}).get('InstanceId'),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # Placement Groups
    try:
        response = ec2.describe_placement_groups()
        for pg in response.get('PlacementGroups', []):
            tags = pg.get('Tags', [])
            resources.append({
                'service': 'ec2',
                'type': 'placement-group',
                'id': pg.get('GroupId', pg['GroupName']),
                'arn': f"arn:aws:ec2:{region}:{account_id}:placement-group/{pg['GroupName']}",
                'name': pg['GroupName'],
                'region': region,
                'details': {
                    'state': pg.get('State'),
                    'strategy': pg.get('Strategy'),
                    'partition_count': pg.get('PartitionCount'),
                },
                'tags': tags_to_dict(tags)
            })
    except Exception:
        pass

    return resources



"""
AWS DataSync resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_datasync_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS DataSync resources: agents, locations, and tasks.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    datasync = session.client('datasync', region_name=region)

    # Agents
    try:
        paginator = datasync.get_paginator('list_agents')
        for page in paginator.paginate():
            for agent in page.get('Agents', []):
                agent_arn = agent['AgentArn']
                agent_name = agent.get('Name', agent_arn.split('/')[-1])

                details = {
                    'status': agent.get('Status'),
                    'platform': agent.get('Platform', {}).get('Version'),
                }

                # Get full agent details
                try:
                    agent_detail = datasync.describe_agent(AgentArn=agent_arn)
                    details['vpc_endpoint_id'] = agent_detail.get('VpcEndpointId')
                    details['private_link_config'] = agent_detail.get('PrivateLinkConfig')
                    details['created_time'] = str(agent_detail.get('CreationTime', ''))
                    details['last_connection_time'] = str(agent_detail.get('LastConnectionTime', ''))
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_paginator = datasync.get_paginator('list_tags_for_resource')
                    for tag_page in tag_paginator.paginate(ResourceArn=agent_arn):
                        for tag in tag_page.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'datasync',
                    'type': 'agent',
                    'id': agent_arn.split('/')[-1],
                    'arn': agent_arn,
                    'name': agent_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Locations
    try:
        paginator = datasync.get_paginator('list_locations')
        for page in paginator.paginate():
            for location in page.get('Locations', []):
                location_arn = location['LocationArn']
                location_uri = location.get('LocationUri', '')

                details = {
                    'location_uri': location_uri,
                }

                # Determine location type from ARN
                location_type = 'unknown'
                if '/loc-' in location_arn:
                    # Try to get more details based on URI prefix
                    if location_uri.startswith('s3://'):
                        location_type = 's3'
                    elif location_uri.startswith('efs://'):
                        location_type = 'efs'
                    elif location_uri.startswith('nfs://'):
                        location_type = 'nfs'
                    elif location_uri.startswith('smb://'):
                        location_type = 'smb'
                    elif location_uri.startswith('fsxw://'):
                        location_type = 'fsx-windows'
                    elif location_uri.startswith('fsxl://'):
                        location_type = 'fsx-lustre'
                    elif location_uri.startswith('fsxo://'):
                        location_type = 'fsx-ontap'
                    elif location_uri.startswith('fsxz://'):
                        location_type = 'fsx-openzfs'
                    elif location_uri.startswith('hdfs://'):
                        location_type = 'hdfs'
                    elif 'blob.core.windows.net' in location_uri:
                        location_type = 'azure-blob'

                details['location_type'] = location_type

                # Get tags
                tags = {}
                try:
                    tag_paginator = datasync.get_paginator('list_tags_for_resource')
                    for tag_page in tag_paginator.paginate(ResourceArn=location_arn):
                        for tag in tag_page.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'datasync',
                    'type': 'location',
                    'id': location_arn.split('/')[-1],
                    'arn': location_arn,
                    'name': location_uri or location_arn.split('/')[-1],
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Tasks
    try:
        paginator = datasync.get_paginator('list_tasks')
        for page in paginator.paginate():
            for task in page.get('Tasks', []):
                task_arn = task['TaskArn']
                task_name = task.get('Name', task_arn.split('/')[-1])

                details = {
                    'status': task.get('Status'),
                }

                # Get full task details
                try:
                    task_detail = datasync.describe_task(TaskArn=task_arn)
                    details['source_location_arn'] = task_detail.get('SourceLocationArn')
                    details['destination_location_arn'] = task_detail.get('DestinationLocationArn')
                    details['cloud_watch_log_group_arn'] = task_detail.get('CloudWatchLogGroupArn')
                    details['created_time'] = str(task_detail.get('CreationTime', ''))
                    details['current_task_execution_arn'] = task_detail.get('CurrentTaskExecutionArn')

                    options = task_detail.get('Options', {})
                    if options:
                        details['verify_mode'] = options.get('VerifyMode')
                        details['overwrite_mode'] = options.get('OverwriteMode')
                        details['transfer_mode'] = options.get('TransferMode')
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_paginator = datasync.get_paginator('list_tags_for_resource')
                    for tag_page in tag_paginator.paginate(ResourceArn=task_arn):
                        for tag in tag_page.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'datasync',
                    'type': 'task',
                    'id': task_arn.split('/')[-1],
                    'arn': task_arn,
                    'name': task_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
Amazon Fraud Detector resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


# Fraud Detector supported regions
FRAUDDETECTOR_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-2',
    'eu-west-1',
    'ap-southeast-1', 'ap-southeast-2',
}


def collect_frauddetector_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Amazon Fraud Detector resources: detectors, models, event types.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in FRAUDDETECTOR_REGIONS:
        return []

    resources = []
    frauddetector = session.client('frauddetector', region_name=region)

    # Detectors
    try:
        response = frauddetector.get_detectors()
        for detector in response.get('detectors', []):
            detector_id = detector.get('detectorId', '')
            detector_arn = detector.get('arn', f"arn:aws:frauddetector:{region}:{account_id}:detector/{detector_id}")

            details = {
                'description': detector.get('description'),
                'event_type_name': detector.get('eventTypeName'),
                'last_updated_time': detector.get('lastUpdatedTime'),
                'created_time': detector.get('createdTime'),
            }

            resources.append({
                'service': 'frauddetector',
                'type': 'detector',
                'id': detector_id,
                'arn': detector_arn,
                'name': detector_id,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    # Models
    try:
        response = frauddetector.get_models()
        for model in response.get('models', []):
            model_id = model.get('modelId', '')
            model_arn = model.get('arn', f"arn:aws:frauddetector:{region}:{account_id}:model/{model_id}")

            details = {
                'model_type': model.get('modelType'),
                'description': model.get('description'),
                'event_type_name': model.get('eventTypeName'),
                'created_time': model.get('createdTime'),
                'last_updated_time': model.get('lastUpdatedTime'),
            }

            resources.append({
                'service': 'frauddetector',
                'type': 'model',
                'id': model_id,
                'arn': model_arn,
                'name': model_id,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    # Event Types
    try:
        response = frauddetector.get_event_types()
        for event_type in response.get('eventTypes', []):
            event_type_name = event_type.get('name', '')
            event_type_arn = event_type.get('arn', f"arn:aws:frauddetector:{region}:{account_id}:event-type/{event_type_name}")

            details = {
                'description': event_type.get('description'),
                'event_variables': event_type.get('eventVariables'),
                'labels': event_type.get('labels'),
                'entity_types': event_type.get('entityTypes'),
                'created_time': event_type.get('createdTime'),
                'last_updated_time': event_type.get('lastUpdatedTime'),
            }

            resources.append({
                'service': 'frauddetector',
                'type': 'event-type',
                'id': event_type_name,
                'arn': event_type_arn,
                'name': event_type_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    return resources



"""
AWS Config resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_config_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Config resources: configuration recorders, delivery channels, rules, aggregators.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    config = session.client('config', region_name=region)

    # Configuration Recorders
    try:
        response = config.describe_configuration_recorders()
        for recorder in response.get('ConfigurationRecorders', []):
            recorder_name = recorder['name']

            # Get recorder status
            status = 'unknown'
            try:
                status_response = config.describe_configuration_recorder_status(
                    ConfigurationRecorderNames=[recorder_name]
                )
                statuses = status_response.get('ConfigurationRecordersStatus', [])
                if statuses:
                    status = 'recording' if statuses[0].get('recording') else 'stopped'
            except Exception:
                pass

            resources.append({
                'service': 'config',
                'type': 'configuration-recorder',
                'id': recorder_name,
                'arn': f"arn:aws:config:{region}:{account_id}:config-recorder/{recorder_name}",
                'name': recorder_name,
                'region': region,
                'details': {
                    'role_arn': recorder.get('roleARN'),
                    'recording_group': recorder.get('recordingGroup'),
                    'status': status,
                },
                'tags': {}
            })
    except Exception:
        pass

    # Delivery Channels
    try:
        response = config.describe_delivery_channels()
        for channel in response.get('DeliveryChannels', []):
            channel_name = channel['name']

            resources.append({
                'service': 'config',
                'type': 'delivery-channel',
                'id': channel_name,
                'arn': f"arn:aws:config:{region}:{account_id}:delivery-channel/{channel_name}",
                'name': channel_name,
                'region': region,
                'details': {
                    's3_bucket_name': channel.get('s3BucketName'),
                    's3_key_prefix': channel.get('s3KeyPrefix'),
                    's3_kms_key_arn': channel.get('s3KmsKeyArn'),
                    'sns_topic_arn': channel.get('snsTopicARN'),
                    'config_snapshot_delivery_properties': channel.get('configSnapshotDeliveryProperties'),
                },
                'tags': {}
            })
    except Exception:
        pass

    # Config Rules
    try:
        paginator = config.get_paginator('describe_config_rules')
        for page in paginator.paginate():
            for rule in page.get('ConfigRules', []):
                rule_name = rule['ConfigRuleName']
                rule_arn = rule['ConfigRuleArn']

                # Get tags
                tags = {}
                try:
                    tag_response = config.list_tags_for_resource(ResourceArn=rule_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'config',
                    'type': 'config-rule',
                    'id': rule_name,
                    'arn': rule_arn,
                    'name': rule_name,
                    'region': region,
                    'details': {
                        'config_rule_state': rule.get('ConfigRuleState'),
                        'description': rule.get('Description'),
                        'scope': rule.get('Scope'),
                        'source': rule.get('Source', {}).get('Owner'),
                        'source_identifier': rule.get('Source', {}).get('SourceIdentifier'),
                        'maximum_execution_frequency': rule.get('MaximumExecutionFrequency'),
                        'created_by': rule.get('CreatedBy'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Configuration Aggregators
    try:
        paginator = config.get_paginator('describe_configuration_aggregators')
        for page in paginator.paginate():
            for aggregator in page.get('ConfigurationAggregators', []):
                agg_name = aggregator['ConfigurationAggregatorName']
                agg_arn = aggregator['ConfigurationAggregatorArn']

                # Get tags
                tags = {}
                try:
                    tag_response = config.list_tags_for_resource(ResourceArn=agg_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'config',
                    'type': 'configuration-aggregator',
                    'id': agg_name,
                    'arn': agg_arn,
                    'name': agg_name,
                    'region': region,
                    'details': {
                        'account_aggregation_sources': aggregator.get('AccountAggregationSources'),
                        'organization_aggregation_source': aggregator.get('OrganizationAggregationSource'),
                        'creation_time': str(aggregator.get('CreationTime', '')),
                        'last_updated_time': str(aggregator.get('LastUpdatedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Conformance Packs
    try:
        paginator = config.get_paginator('describe_conformance_packs')
        for page in paginator.paginate():
            for pack in page.get('ConformancePackDetails', []):
                pack_name = pack['ConformancePackName']
                pack_arn = pack['ConformancePackArn']

                resources.append({
                    'service': 'config',
                    'type': 'conformance-pack',
                    'id': pack_name,
                    'arn': pack_arn,
                    'name': pack_name,
                    'region': region,
                    'details': {
                        'delivery_s3_bucket': pack.get('DeliveryS3Bucket'),
                        'delivery_s3_key_prefix': pack.get('DeliveryS3KeyPrefix'),
                        'created_by': pack.get('CreatedBy'),
                        'last_update_requested_time': str(pack.get('LastUpdateRequestedTime', '')),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Simple Email Service v2 (SESv2) resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_sesv2_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS SESv2 resources: email identities, configuration sets, contact lists, templates, and IP pools.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    ses = session.client('sesv2', region_name=region)

    # Email Identities
    try:
        paginator = ses.get_paginator('list_email_identities')
        for page in paginator.paginate():
            for identity in page.get('EmailIdentities', []):
                identity_name = identity['IdentityName']
                identity_type = identity.get('IdentityType', 'UNKNOWN')

                details = {
                    'identity_type': identity_type,
                    'sending_enabled': identity.get('SendingEnabled'),
                    'verification_status': identity.get('VerificationStatus'),
                }

                # Get additional details
                try:
                    identity_info = ses.get_email_identity(EmailIdentity=identity_name)
                    details['dkim_attributes'] = {
                        'signing_enabled': identity_info.get('DkimAttributes', {}).get('SigningEnabled'),
                        'status': identity_info.get('DkimAttributes', {}).get('Status'),
                        'signing_attributes_origin': identity_info.get('DkimAttributes', {}).get('SigningAttributesOrigin'),
                    }
                    details['mail_from_attributes'] = {
                        'mail_from_domain': identity_info.get('MailFromAttributes', {}).get('MailFromDomain'),
                        'mail_from_domain_status': identity_info.get('MailFromAttributes', {}).get('MailFromDomainStatus'),
                    }
                    details['policies'] = list(identity_info.get('Policies', {}).keys())
                    details['configuration_set_name'] = identity_info.get('ConfigurationSetName')
                    tags = {t['Key']: t['Value'] for t in identity_info.get('Tags', [])}
                except Exception:
                    tags = {}

                resources.append({
                    'service': 'sesv2',
                    'type': 'email-identity',
                    'id': identity_name,
                    'arn': f"arn:aws:ses:{region}:{account_id}:identity/{identity_name}",
                    'name': identity_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Configuration Sets
    try:
        paginator = ses.get_paginator('list_configuration_sets')
        for page in paginator.paginate():
            for config_set in page.get('ConfigurationSets', []):
                config_set_name = config_set

                details = {}

                # Get additional details
                try:
                    cs_info = ses.get_configuration_set(ConfigurationSetName=config_set_name)
                    details['tracking_options'] = cs_info.get('TrackingOptions', {}).get('CustomRedirectDomain')
                    details['delivery_options'] = {
                        'tls_policy': cs_info.get('DeliveryOptions', {}).get('TlsPolicy'),
                        'sending_pool_name': cs_info.get('DeliveryOptions', {}).get('SendingPoolName'),
                    }
                    details['reputation_options'] = {
                        'reputation_metrics_enabled': cs_info.get('ReputationOptions', {}).get('ReputationMetricsEnabled'),
                        'last_fresh_start': str(cs_info.get('ReputationOptions', {}).get('LastFreshStart', '')),
                    }
                    details['sending_options'] = {
                        'sending_enabled': cs_info.get('SendingOptions', {}).get('SendingEnabled'),
                    }
                    details['suppression_options'] = cs_info.get('SuppressionOptions', {}).get('SuppressedReasons', [])
                    tags = {t['Key']: t['Value'] for t in cs_info.get('Tags', [])}
                except Exception:
                    tags = {}

                resources.append({
                    'service': 'sesv2',
                    'type': 'configuration-set',
                    'id': config_set_name,
                    'arn': f"arn:aws:ses:{region}:{account_id}:configuration-set/{config_set_name}",
                    'name': config_set_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Contact Lists
    try:
        paginator = ses.get_paginator('list_contact_lists')
        for page in paginator.paginate():
            for contact_list in page.get('ContactLists', []):
                list_name = contact_list['ContactListName']

                details = {
                    'last_updated_timestamp': str(contact_list.get('LastUpdatedTimestamp', '')),
                }

                # Get additional details
                try:
                    cl_info = ses.get_contact_list(ContactListName=list_name)
                    details['description'] = cl_info.get('Description')
                    details['topics'] = [t.get('TopicName') for t in cl_info.get('Topics', [])]
                    details['created_timestamp'] = str(cl_info.get('CreatedTimestamp', ''))
                    tags = {t['Key']: t['Value'] for t in cl_info.get('Tags', [])}
                except Exception:
                    tags = {}

                resources.append({
                    'service': 'sesv2',
                    'type': 'contact-list',
                    'id': list_name,
                    'arn': f"arn:aws:ses:{region}:{account_id}:contact-list/{list_name}",
                    'name': list_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Email Templates
    try:
        paginator = ses.get_paginator('list_email_templates')
        for page in paginator.paginate():
            for template in page.get('TemplatesMetadata', []):
                template_name = template['TemplateName']

                details = {
                    'created_timestamp': str(template.get('CreatedTimestamp', '')),
                }

                resources.append({
                    'service': 'sesv2',
                    'type': 'email-template',
                    'id': template_name,
                    'arn': f"arn:aws:ses:{region}:{account_id}:template/{template_name}",
                    'name': template_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Dedicated IP Pools
    try:
        paginator = ses.get_paginator('list_dedicated_ip_pools')
        for page in paginator.paginate():
            for pool_name in page.get('DedicatedIpPools', []):
                details = {}

                # Get pool details
                try:
                    pool_info = ses.get_dedicated_ip_pool(PoolName=pool_name)
                    pool_data = pool_info.get('DedicatedIpPool', {})
                    details['scaling_mode'] = pool_data.get('ScalingMode')
                except Exception:
                    pass

                resources.append({
                    'service': 'sesv2',
                    'type': 'dedicated-ip-pool',
                    'id': pool_name,
                    'arn': f"arn:aws:ses:{region}:{account_id}:dedicated-ip-pool/{pool_name}",
                    'name': pool_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Network Firewall resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_network_firewall_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Network Firewall resources: firewalls, policies, rule groups,
    and TLS inspection configurations.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    nfw = session.client('network-firewall', region_name=region)

    # Firewalls
    try:
        paginator = nfw.get_paginator('list_firewalls')
        for page in paginator.paginate():
            for fw in page.get('Firewalls', []):
                fw_name = fw.get('FirewallName')
                fw_arn = fw.get('FirewallArn', f"arn:aws:network-firewall:{region}:{account_id}:firewall/{fw_name}")

                details = {}
                tags = {}

                # Get detailed firewall info
                try:
                    fw_response = nfw.describe_firewall(FirewallArn=fw_arn)
                    firewall = fw_response.get('Firewall', {})
                    fw_status = fw_response.get('FirewallStatus', {})

                    details = {
                        'firewall_id': firewall.get('FirewallId'),
                        'firewall_policy_arn': firewall.get('FirewallPolicyArn'),
                        'vpc_id': firewall.get('VpcId'),
                        'subnet_mappings': [s.get('SubnetId') for s in firewall.get('SubnetMappings', [])],
                        'delete_protection': firewall.get('DeleteProtection'),
                        'subnet_change_protection': firewall.get('SubnetChangeProtection'),
                        'firewall_policy_change_protection': firewall.get('FirewallPolicyChangeProtection'),
                        'description': firewall.get('Description'),
                        'encryption_configuration': firewall.get('EncryptionConfiguration', {}).get('Type'),
                        'status': fw_status.get('Status'),
                        'configuration_sync_state': fw_status.get('ConfigurationSyncStateSummary'),
                    }

                    # Get sync states per AZ
                    sync_states = fw_status.get('SyncStates', {})
                    if sync_states:
                        details['availability_zones'] = list(sync_states.keys())

                    tags = {t['Key']: t['Value'] for t in firewall.get('Tags', [])}

                except Exception:
                    pass

                # Get logging configuration
                try:
                    log_response = nfw.describe_logging_configuration(FirewallArn=fw_arn)
                    log_config = log_response.get('LoggingConfiguration', {})
                    log_destinations = log_config.get('LogDestinationConfigs', [])
                    if log_destinations:
                        details['logging_types'] = [d.get('LogType') for d in log_destinations]
                        details['logging_destinations'] = [d.get('LogDestinationType') for d in log_destinations]
                except Exception:
                    pass

                resources.append({
                    'service': 'network-firewall',
                    'type': 'firewall',
                    'id': fw_name,
                    'arn': fw_arn,
                    'name': fw_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Firewall Policies
    try:
        paginator = nfw.get_paginator('list_firewall_policies')
        for page in paginator.paginate():
            for policy in page.get('FirewallPolicies', []):
                policy_name = policy.get('Name')
                policy_arn = policy.get('Arn', f"arn:aws:network-firewall:{region}:{account_id}:firewall-policy/{policy_name}")

                details = {}
                tags = {}

                # Get detailed policy info
                try:
                    policy_response = nfw.describe_firewall_policy(FirewallPolicyArn=policy_arn)
                    policy_detail = policy_response.get('FirewallPolicy', {})
                    policy_metadata = policy_response.get('FirewallPolicyResponse', {})

                    details = {
                        'policy_id': policy_metadata.get('FirewallPolicyId'),
                        'description': policy_metadata.get('Description'),
                        'status': policy_metadata.get('FirewallPolicyStatus'),
                        'number_of_associations': policy_metadata.get('NumberOfAssociations'),
                        'encryption_configuration': policy_metadata.get('EncryptionConfiguration', {}).get('Type'),
                        'stateless_default_actions': policy_detail.get('StatelessDefaultActions', []),
                        'stateless_fragment_default_actions': policy_detail.get('StatelessFragmentDefaultActions', []),
                        'stateful_engine_options': policy_detail.get('StatefulEngineOptions', {}).get('RuleOrder'),
                        'stateless_rule_group_count': len(policy_detail.get('StatelessRuleGroupReferences', [])),
                        'stateful_rule_group_count': len(policy_detail.get('StatefulRuleGroupReferences', [])),
                    }

                    tags = {t['Key']: t['Value'] for t in policy_metadata.get('Tags', [])}

                except Exception:
                    pass

                resources.append({
                    'service': 'network-firewall',
                    'type': 'firewall-policy',
                    'id': policy_name,
                    'arn': policy_arn,
                    'name': policy_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Rule Groups
    try:
        paginator = nfw.get_paginator('list_rule_groups')
        for page in paginator.paginate():
            for rg in page.get('RuleGroups', []):
                rg_name = rg.get('Name')
                rg_arn = rg.get('Arn', f"arn:aws:network-firewall:{region}:{account_id}:rule-group/{rg_name}")

                details = {}
                tags = {}

                # Get detailed rule group info
                try:
                    rg_response = nfw.describe_rule_group(RuleGroupArn=rg_arn)
                    rg_metadata = rg_response.get('RuleGroupResponse', {})

                    details = {
                        'rule_group_id': rg_metadata.get('RuleGroupId'),
                        'description': rg_metadata.get('Description'),
                        'type': rg_metadata.get('Type'),
                        'capacity': rg_metadata.get('Capacity'),
                        'status': rg_metadata.get('RuleGroupStatus'),
                        'number_of_associations': rg_metadata.get('NumberOfAssociations'),
                        'encryption_configuration': rg_metadata.get('EncryptionConfiguration', {}).get('Type'),
                        'source_metadata': rg_metadata.get('SourceMetadata', {}).get('SourceArn'),
                        'sns_topic': rg_metadata.get('SnsTopic'),
                        'last_modified_time': str(rg_metadata.get('LastModifiedTime', '')),
                    }

                    # Get analysis results if available
                    analysis = rg_metadata.get('AnalysisResults', [])
                    if analysis:
                        details['analysis_results_count'] = len(analysis)

                    tags = {t['Key']: t['Value'] for t in rg_metadata.get('Tags', [])}

                except Exception:
                    pass

                resources.append({
                    'service': 'network-firewall',
                    'type': 'rule-group',
                    'id': rg_name,
                    'arn': rg_arn,
                    'name': rg_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # TLS Inspection Configurations
    try:
        paginator = nfw.get_paginator('list_tls_inspection_configurations')
        for page in paginator.paginate():
            for tls in page.get('TLSInspectionConfigurations', []):
                tls_name = tls.get('Name')
                tls_arn = tls.get('Arn', f"arn:aws:network-firewall:{region}:{account_id}:tls-configuration/{tls_name}")

                details = {}
                tags = {}

                # Get detailed TLS config
                try:
                    tls_response = nfw.describe_tls_inspection_configuration(
                        TLSInspectionConfigurationArn=tls_arn
                    )
                    tls_metadata = tls_response.get('TLSInspectionConfigurationResponse', {})

                    details = {
                        'tls_inspection_configuration_id': tls_metadata.get('TLSInspectionConfigurationId'),
                        'description': tls_metadata.get('Description'),
                        'status': tls_metadata.get('TLSInspectionConfigurationStatus'),
                        'number_of_associations': tls_metadata.get('NumberOfAssociations'),
                        'encryption_configuration': tls_metadata.get('EncryptionConfiguration', {}).get('Type'),
                        'last_modified_time': str(tls_metadata.get('LastModifiedTime', '')),
                    }

                    # Get certificate info
                    certs = tls_metadata.get('Certificates', [])
                    if certs:
                        details['certificates_count'] = len(certs)
                        details['certificate_arns'] = [c.get('CertificateArn') for c in certs]

                    tags = {t['Key']: t['Value'] for t in tls_metadata.get('Tags', [])}

                except Exception:
                    pass

                resources.append({
                    'service': 'network-firewall',
                    'type': 'tls-inspection-configuration',
                    'id': tls_name,
                    'arn': tls_arn,
                    'name': tls_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
Lambda resource collector.
"""

import asyncio

import aiobotocore.session
import boto3
from typing import List, Dict, Any, Optional


def collect_lambda__resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Lambda resources: functions, layers, event source mappings.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    lambda_client = session.client('lambda', region_name=region)

    # Collect functions
    functions = []
    try:
        paginator = lambda_client.get_paginator('list_functions')
        for page in paginator.paginate():
            functions.extend(page.get('Functions', []))
    except Exception:
        pass

    # Fetch tags using asyncio
    tags_map = {}
    if functions:
        profile = session.profile_name
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            tags_map = loop.run_until_complete(_fetch_tags_async(profile, region, [f['FunctionArn'] for f in functions]))
        except Exception:
            pass
        finally:
            loop.close()

    # Build function resources
    for func in functions:
        func_name = func['FunctionName']
        func_arn = func['FunctionArn']

        resources.append({
            'service': 'lambda',
            'type': 'function',
            'id': func_name,
            'arn': func_arn,
            'name': func_name,
            'region': region,
            'details': {
                'runtime': func.get('Runtime'),
                'handler': func.get('Handler'),
                'code_size': func.get('CodeSize'),
                'memory_size': func.get('MemorySize'),
                'timeout': func.get('Timeout'),
                'last_modified': func.get('LastModified'),
                'description': func.get('Description'),
                'role': func.get('Role'),
                'vpc_id': func.get('VpcConfig', {}).get('VpcId'),
                'architectures': func.get('Architectures', []),
                'package_type': func.get('PackageType'),
                'ephemeral_storage': func.get('EphemeralStorage', {}).get('Size'),
            },
            'tags': tags_map.get(func_arn, {})
        })

    # Lambda Layers
    try:
        paginator = lambda_client.get_paginator('list_layers')
        for page in paginator.paginate():
            for layer in page.get('Layers', []):
                layer_name = layer['LayerName']
                layer_arn = layer['LayerArn']
                latest_version = layer.get('LatestMatchingVersion', {})

                resources.append({
                    'service': 'lambda',
                    'type': 'layer',
                    'id': layer_name,
                    'arn': layer_arn,
                    'name': layer_name,
                    'region': region,
                    'details': {
                        'latest_version': latest_version.get('Version'),
                        'latest_version_arn': latest_version.get('LayerVersionArn'),
                        'description': latest_version.get('Description'),
                        'created_date': latest_version.get('CreatedDate'),
                        'compatible_runtimes': latest_version.get('CompatibleRuntimes', []),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Event Source Mappings
    try:
        paginator = lambda_client.get_paginator('list_event_source_mappings')
        for page in paginator.paginate():
            for esm in page.get('EventSourceMappings', []):
                esm_uuid = esm['UUID']
                func_arn = esm.get('FunctionArn', '')
                func_name = func_arn.split(':')[-1] if func_arn else 'unknown'

                resources.append({
                    'service': 'lambda',
                    'type': 'event-source-mapping',
                    'id': esm_uuid,
                    'arn': f"arn:aws:lambda:{region}:{account_id}:event-source-mapping:{esm_uuid}",
                    'name': f"{func_name}-{esm_uuid[:8]}",
                    'region': region,
                    'details': {
                        'function_arn': func_arn,
                        'event_source_arn': esm.get('EventSourceArn'),
                        'state': esm.get('State'),
                        'batch_size': esm.get('BatchSize'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    return resources


async def _fetch_tags_async(profile: str, region: str, arns: List[str]) -> Dict[str, Dict[str, str]]:
    """Fetch tags for all functions using asyncio."""
    session = aiobotocore.session.AioSession(profile=profile)

    async with session.create_client('lambda', region_name=region) as client:
        async def get_tags(arn):
            try:
                resp = await client.list_tags(Resource=arn)
                return arn, resp.get('Tags', {})
            except Exception:
                return arn, {}

        tasks = [get_tags(arn) for arn in arns]
        results = await asyncio.gather(*tasks)
        return dict(results)



"""
ACM resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_acm_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect ACM resources: certificates.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    acm = session.client('acm', region_name=region)

    try:
        paginator = acm.get_paginator('list_certificates')
        for page in paginator.paginate():
            for cert_summary in page.get('CertificateSummaryList', []):
                cert_arn = cert_summary['CertificateArn']

                try:
                    # Get certificate details
                    cert_response = acm.describe_certificate(CertificateArn=cert_arn)
                    cert = cert_response.get('Certificate', {})

                    # Get tags
                    tags = {}
                    try:
                        tag_response = acm.list_tags_for_certificate(CertificateArn=cert_arn)
                        for tag in tag_response.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                    except Exception:
                        pass

                    domain_name = cert.get('DomainName', '')

                    resources.append({
                        'service': 'acm',
                        'type': 'certificate',
                        'id': cert_arn.split('/')[-1],
                        'arn': cert_arn,
                        'name': tags.get('Name') or domain_name,
                        'region': region,
                        'details': {
                            'domain_name': domain_name,
                            'subject_alternative_names': cert.get('SubjectAlternativeNames', []),
                            'status': cert.get('Status'),
                            'type': cert.get('Type'),
                            'key_algorithm': cert.get('KeyAlgorithm'),
                            'issuer': cert.get('Issuer'),
                            'created_at': str(cert.get('CreatedAt', '')),
                            'issued_at': str(cert.get('IssuedAt', '')) if cert.get('IssuedAt') else None,
                            'not_before': str(cert.get('NotBefore', '')) if cert.get('NotBefore') else None,
                            'not_after': str(cert.get('NotAfter', '')) if cert.get('NotAfter') else None,
                            'in_use_by': cert.get('InUseBy', []),
                            'renewal_eligibility': cert.get('RenewalEligibility'),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
VPC resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional

from aws_inventory.collector import tags_to_dict, get_tag_value


def collect_vpc_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect VPC resources: VPCs, subnets, route tables, internet gateways,
    NAT gateways, VPC endpoints, VPC peering, transit gateways.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    ec2 = session.client('ec2', region_name=region)

    # VPCs
    try:
        paginator = ec2.get_paginator('describe_vpcs')
        for page in paginator.paginate():
            for vpc in page.get('Vpcs', []):
                tags = vpc.get('Tags', [])
                resources.append({
                    'service': 'vpc',
                    'type': 'vpc',
                    'id': vpc['VpcId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:vpc/{vpc['VpcId']}",
                    'name': get_tag_value(tags, 'Name') or vpc['VpcId'],
                    'region': region,
                    'is_default': vpc.get('IsDefault', False),
                    'details': {
                        'cidr_block': vpc.get('CidrBlock'),
                        'state': vpc.get('State'),
                        'is_default': vpc.get('IsDefault'),
                        'dhcp_options_id': vpc.get('DhcpOptionsId'),
                        'instance_tenancy': vpc.get('InstanceTenancy'),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # Build set of default VPC IDs and their DHCP options for marking child resources
    default_vpc_ids = {r['id'] for r in resources if r.get('is_default')}
    default_dhcp_ids = {r['details']['dhcp_options_id'] for r in resources if r.get('is_default') and r['details'].get('dhcp_options_id')}

    # Subnets
    try:
        paginator = ec2.get_paginator('describe_subnets')
        for page in paginator.paginate():
            for subnet in page.get('Subnets', []):
                tags = subnet.get('Tags', [])
                resources.append({
                    'service': 'vpc',
                    'type': 'subnet',
                    'id': subnet['SubnetId'],
                    'arn': subnet.get('SubnetArn', f"arn:aws:ec2:{region}:{account_id}:subnet/{subnet['SubnetId']}"),
                    'name': get_tag_value(tags, 'Name') or subnet['SubnetId'],
                    'region': region,
                    'is_default': subnet.get('DefaultForAz', False),
                    'details': {
                        'vpc_id': subnet.get('VpcId'),
                        'cidr_block': subnet.get('CidrBlock'),
                        'availability_zone': subnet.get('AvailabilityZone'),
                        'state': subnet.get('State'),
                        'available_ip_count': subnet.get('AvailableIpAddressCount'),
                        'map_public_ip_on_launch': subnet.get('MapPublicIpOnLaunch'),
                        'default_for_az': subnet.get('DefaultForAz'),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # Route Tables
    try:
        paginator = ec2.get_paginator('describe_route_tables')
        for page in paginator.paginate():
            for rt in page.get('RouteTables', []):
                tags = rt.get('Tags', [])
                is_main = any(a.get('Main') for a in rt.get('Associations', []))
                resources.append({
                    'service': 'vpc',
                    'type': 'route-table',
                    'id': rt['RouteTableId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:route-table/{rt['RouteTableId']}",
                    'name': get_tag_value(tags, 'Name') or rt['RouteTableId'],
                    'region': region,
                    'is_default': rt.get('VpcId') in default_vpc_ids and is_main,
                    'details': {
                        'vpc_id': rt.get('VpcId'),
                        'routes_count': len(rt.get('Routes', [])),
                        'associations_count': len(rt.get('Associations', [])),
                        'main': is_main,
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # Internet Gateways
    try:
        paginator = ec2.get_paginator('describe_internet_gateways')
        for page in paginator.paginate():
            for igw in page.get('InternetGateways', []):
                tags = igw.get('Tags', [])
                attachments = igw.get('Attachments', [])
                resources.append({
                    'service': 'vpc',
                    'type': 'internet-gateway',
                    'id': igw['InternetGatewayId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:internet-gateway/{igw['InternetGatewayId']}",
                    'name': get_tag_value(tags, 'Name') or igw['InternetGatewayId'],
                    'region': region,
                    'is_default': (attachments[0].get('VpcId') in default_vpc_ids) if attachments else False,
                    'details': {
                        'vpc_id': attachments[0].get('VpcId') if attachments else None,
                        'state': attachments[0].get('State') if attachments else 'detached',
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # NAT Gateways
    try:
        paginator = ec2.get_paginator('describe_nat_gateways')
        for page in paginator.paginate():
            for nat in page.get('NatGateways', []):
                tags = nat.get('Tags', [])

                # Skip deleted NAT gateways
                if nat.get('State') == 'deleted':
                    continue

                resources.append({
                    'service': 'vpc',
                    'type': 'nat-gateway',
                    'id': nat['NatGatewayId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:natgateway/{nat['NatGatewayId']}",
                    'name': get_tag_value(tags, 'Name') or nat['NatGatewayId'],
                    'region': region,
                    'details': {
                        'vpc_id': nat.get('VpcId'),
                        'subnet_id': nat.get('SubnetId'),
                        'state': nat.get('State'),
                        'connectivity_type': nat.get('ConnectivityType'),
                        'public_ip': nat.get('NatGatewayAddresses', [{}])[0].get('PublicIp') if nat.get('NatGatewayAddresses') else None,
                        'private_ip': nat.get('NatGatewayAddresses', [{}])[0].get('PrivateIp') if nat.get('NatGatewayAddresses') else None,
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # VPC Endpoints
    try:
        paginator = ec2.get_paginator('describe_vpc_endpoints')
        for page in paginator.paginate():
            for endpoint in page.get('VpcEndpoints', []):
                tags = endpoint.get('Tags', [])
                resources.append({
                    'service': 'vpc',
                    'type': 'vpc-endpoint',
                    'id': endpoint['VpcEndpointId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:vpc-endpoint/{endpoint['VpcEndpointId']}",
                    'name': get_tag_value(tags, 'Name') or endpoint['VpcEndpointId'],
                    'region': region,
                    'details': {
                        'vpc_id': endpoint.get('VpcId'),
                        'service_name': endpoint.get('ServiceName'),
                        'endpoint_type': endpoint.get('VpcEndpointType'),
                        'state': endpoint.get('State'),
                        'private_dns_enabled': endpoint.get('PrivateDnsEnabled'),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # VPC Peering Connections
    try:
        paginator = ec2.get_paginator('describe_vpc_peering_connections')
        for page in paginator.paginate():
            for pcx in page.get('VpcPeeringConnections', []):
                tags = pcx.get('Tags', [])

                # Skip deleted peering connections
                status = pcx.get('Status', {}).get('Code', '')
                if status in ['deleted', 'rejected', 'failed']:
                    continue

                resources.append({
                    'service': 'vpc',
                    'type': 'vpc-peering',
                    'id': pcx['VpcPeeringConnectionId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:vpc-peering-connection/{pcx['VpcPeeringConnectionId']}",
                    'name': get_tag_value(tags, 'Name') or pcx['VpcPeeringConnectionId'],
                    'region': region,
                    'details': {
                        'status': status,
                        'requester_vpc_id': pcx.get('RequesterVpcInfo', {}).get('VpcId'),
                        'requester_cidr': pcx.get('RequesterVpcInfo', {}).get('CidrBlock'),
                        'accepter_vpc_id': pcx.get('AccepterVpcInfo', {}).get('VpcId'),
                        'accepter_cidr': pcx.get('AccepterVpcInfo', {}).get('CidrBlock'),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # Transit Gateways (owned by this account)
    try:
        paginator = ec2.get_paginator('describe_transit_gateways')
        for page in paginator.paginate():
            for tgw in page.get('TransitGateways', []):
                # Only include transit gateways owned by this account
                if tgw.get('OwnerId') != account_id:
                    continue

                tags = tgw.get('Tags', [])
                resources.append({
                    'service': 'vpc',
                    'type': 'transit-gateway',
                    'id': tgw['TransitGatewayId'],
                    'arn': tgw.get('TransitGatewayArn', f"arn:aws:ec2:{region}:{account_id}:transit-gateway/{tgw['TransitGatewayId']}"),
                    'name': get_tag_value(tags, 'Name') or tgw['TransitGatewayId'],
                    'region': region,
                    'details': {
                        'state': tgw.get('State'),
                        'description': tgw.get('Description'),
                        'amazon_side_asn': tgw.get('Options', {}).get('AmazonSideAsn'),
                        'auto_accept_shared_attachments': tgw.get('Options', {}).get('AutoAcceptSharedAttachments'),
                        'default_route_table_association': tgw.get('Options', {}).get('DefaultRouteTableAssociation'),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # Transit Gateway Attachments
    try:
        paginator = ec2.get_paginator('describe_transit_gateway_attachments')
        for page in paginator.paginate():
            for attach in page.get('TransitGatewayAttachments', []):
                tags = attach.get('Tags', [])

                # Skip deleted attachments
                if attach.get('State') == 'deleted':
                    continue

                resources.append({
                    'service': 'vpc',
                    'type': 'transit-gateway-attachment',
                    'id': attach['TransitGatewayAttachmentId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:transit-gateway-attachment/{attach['TransitGatewayAttachmentId']}",
                    'name': get_tag_value(tags, 'Name') or attach['TransitGatewayAttachmentId'],
                    'region': region,
                    'details': {
                        'transit_gateway_id': attach.get('TransitGatewayId'),
                        'resource_type': attach.get('ResourceType'),
                        'resource_id': attach.get('ResourceId'),
                        'state': attach.get('State'),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    # DHCP Options Sets
    try:
        response = ec2.describe_dhcp_options()
        for dhcp in response.get('DhcpOptions', []):
            tags = dhcp.get('Tags', [])
            resources.append({
                'service': 'vpc',
                'type': 'dhcp-options',
                'id': dhcp['DhcpOptionsId'],
                'arn': f"arn:aws:ec2:{region}:{account_id}:dhcp-options/{dhcp['DhcpOptionsId']}",
                'name': get_tag_value(tags, 'Name') or dhcp['DhcpOptionsId'],
                'region': region,
                'is_default': dhcp['DhcpOptionsId'] in default_dhcp_ids,
                'details': {
                    'configurations': {cfg['Key']: cfg.get('Values', []) for cfg in dhcp.get('DhcpConfigurations', [])},
                },
                'tags': tags_to_dict(tags)
            })
    except Exception:
        pass

    # Network ACLs
    try:
        paginator = ec2.get_paginator('describe_network_acls')
        for page in paginator.paginate():
            for nacl in page.get('NetworkAcls', []):
                tags = nacl.get('Tags', [])
                resources.append({
                    'service': 'vpc',
                    'type': 'network-acl',
                    'id': nacl['NetworkAclId'],
                    'arn': f"arn:aws:ec2:{region}:{account_id}:network-acl/{nacl['NetworkAclId']}",
                    'name': get_tag_value(tags, 'Name') or nacl['NetworkAclId'],
                    'region': region,
                    'is_default': nacl.get('IsDefault', False),
                    'details': {
                        'vpc_id': nacl.get('VpcId'),
                        'is_default': nacl.get('IsDefault'),
                        'entries_count': len(nacl.get('Entries', [])),
                        'associations_count': len(nacl.get('Associations', [])),
                    },
                    'tags': tags_to_dict(tags)
                })
    except Exception:
        pass

    return resources



"""
AWS Elemental MediaConvert resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_mediaconvert_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS MediaConvert resources: queues, job templates, and presets.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []

    # First get the endpoint for this region
    try:
        mc_endpoints = session.client('mediaconvert', region_name=region)
        endpoints_response = mc_endpoints.describe_endpoints()
        endpoints = endpoints_response.get('Endpoints', [])
        if not endpoints:
            return resources
        endpoint_url = endpoints[0].get('Url')
        if not endpoint_url:
            return resources
    except Exception:
        return resources

    # Create client with the account-specific endpoint
    mc = session.client('mediaconvert', region_name=region, endpoint_url=endpoint_url)

    # Queues (skip AWS default queue that exists in every region)
    try:
        paginator = mc.get_paginator('list_queues')
        for page in paginator.paginate():
            for queue in page.get('Queues', []):
                queue_name = queue['Name']

                # Skip the AWS default queue
                if queue_name == 'Default':
                    continue

                queue_arn = queue.get('Arn', '')

                details = {
                    'status': queue.get('Status'),
                    'type': queue.get('Type'),
                    'pricing_plan': queue.get('PricingPlan'),
                    'submitted_jobs_count': queue.get('SubmittedJobsCount'),
                    'progressing_jobs_count': queue.get('ProgressingJobsCount'),
                    'created_at': str(queue.get('CreatedAt', '')),
                    'last_updated': str(queue.get('LastUpdated', '')),
                    'description': queue.get('Description'),
                }

                resources.append({
                    'service': 'mediaconvert',
                    'type': 'queue',
                    'id': queue_name,
                    'arn': queue_arn,
                    'name': queue_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Job Templates
    try:
        paginator = mc.get_paginator('list_job_templates')
        for page in paginator.paginate():
            for template in page.get('JobTemplates', []):
                template_name = template['Name']
                template_arn = template.get('Arn', '')

                details = {
                    'type': template.get('Type'),
                    'category': template.get('Category'),
                    'description': template.get('Description'),
                    'created_at': str(template.get('CreatedAt', '')),
                    'last_updated': str(template.get('LastUpdated', '')),
                }

                resources.append({
                    'service': 'mediaconvert',
                    'type': 'job-template',
                    'id': template_name,
                    'arn': template_arn,
                    'name': template_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Presets
    try:
        paginator = mc.get_paginator('list_presets')
        for page in paginator.paginate():
            for preset in page.get('Presets', []):
                preset_name = preset['Name']
                preset_arn = preset.get('Arn', '')

                details = {
                    'type': preset.get('Type'),
                    'category': preset.get('Category'),
                    'description': preset.get('Description'),
                    'created_at': str(preset.get('CreatedAt', '')),
                    'last_updated': str(preset.get('LastUpdated', '')),
                }

                resources.append({
                    'service': 'mediaconvert',
                    'type': 'preset',
                    'id': preset_name,
                    'arn': preset_arn,
                    'name': preset_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Kendra resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# Kendra supported regions (from https://docs.aws.amazon.com/general/latest/gr/kendra.html)
KENDRA_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-2',
    'ap-south-1', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1',
    'ca-central-1',
    'eu-west-1', 'eu-west-2',
}


def collect_kendra_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Kendra resources: indexes and data sources.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in KENDRA_REGIONS:
        return []

    resources = []
    kendra = session.client('kendra', region_name=region)

    # Indexes
    try:
        paginator = kendra.get_paginator('list_indices')
        for page in paginator.paginate():
            for index in page.get('IndexConfigurationSummaryItems', []):
                index_id = index['Id']
                index_name = index.get('Name', index_id)

                details = {
                    'status': index.get('Status'),
                    'edition': index.get('Edition'),
                    'created_at': str(index.get('CreatedAt', '')),
                    'updated_at': str(index.get('UpdatedAt', '')),
                }

                resources.append({
                    'service': 'kendra',
                    'type': 'index',
                    'id': index_id,
                    'arn': f"arn:aws:kendra:{region}:{account_id}:index/{index_id}",
                    'name': index_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })

                # Data Sources for this index
                try:
                    ds_paginator = kendra.get_paginator('list_data_sources')
                    for ds_page in ds_paginator.paginate(IndexId=index_id):
                        for ds in ds_page.get('SummaryItems', []):
                            ds_id = ds['Id']
                            ds_name = ds.get('Name', ds_id)

                            ds_details = {
                                'index_id': index_id,
                                'type': ds.get('Type'),
                                'status': ds.get('Status'),
                                'created_at': str(ds.get('CreatedAt', '')),
                                'updated_at': str(ds.get('UpdatedAt', '')),
                                'language_code': ds.get('LanguageCode'),
                            }

                            resources.append({
                                'service': 'kendra',
                                'type': 'data-source',
                                'id': ds_id,
                                'arn': f"arn:aws:kendra:{region}:{account_id}:index/{index_id}/data-source/{ds_id}",
                                'name': ds_name,
                                'region': region,
                                'details': ds_details,
                                'tags': {}
                            })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
AWS Compute Optimizer resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_computeoptimizer_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Compute Optimizer recommendations: EC2, Auto Scaling, Lambda, EBS, ECS.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    co = session.client('compute-optimizer', region_name=region)

    # Check enrollment status
    try:
        status = co.get_enrollment_status()
        if status.get('status') != 'Active':
            return []
    except Exception:
        return []

    # EC2 Instance Recommendations
    try:
        response = co.get_ec2_instance_recommendations()
        for rec in response.get('instanceRecommendations', []):
            instance_arn = rec.get('instanceArn', '')
            instance_name = rec.get('instanceName', instance_arn.split('/')[-1])

            details = {
                'finding': rec.get('finding'),
                'finding_reason_codes': rec.get('findingReasonCodes', []),
                'current_instance_type': rec.get('currentInstanceType'),
                'look_back_period_in_days': rec.get('lookBackPeriodInDays'),
                'last_refresh_timestamp': str(rec.get('lastRefreshTimestamp', '')),
                'current_performance_risk': rec.get('currentPerformanceRisk'),
                'effective_recommendation_preferences': rec.get('effectiveRecommendationPreferences', {}),
                'inference_accelerator_state': rec.get('inferenceAcceleratorState'),
                'idle': rec.get('idle'),
            }

            # Recommendation options count
            options = rec.get('recommendationOptions', [])
            details['recommendation_options_count'] = len(options)
            if options:
                top_option = options[0]
                details['top_recommendation_instance_type'] = top_option.get('instanceType')
                details['top_recommendation_performance_risk'] = top_option.get('performanceRisk')

            resources.append({
                'service': 'compute-optimizer',
                'type': 'ec2-recommendation',
                'id': instance_arn.split('/')[-1] if instance_arn else instance_name,
                'arn': instance_arn,
                'name': instance_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    # Auto Scaling Group Recommendations
    try:
        response = co.get_auto_scaling_group_recommendations()
        for rec in response.get('autoScalingGroupRecommendations', []):
            asg_arn = rec.get('autoScalingGroupArn', '')
            asg_name = rec.get('autoScalingGroupName', asg_arn.split('/')[-1])

            details = {
                'finding': rec.get('finding'),
                'current_instance_type': rec.get('currentConfiguration', {}).get('instanceType'),
                'current_desired_capacity': rec.get('currentConfiguration', {}).get('desiredCapacity'),
                'current_min_size': rec.get('currentConfiguration', {}).get('minSize'),
                'current_max_size': rec.get('currentConfiguration', {}).get('maxSize'),
                'look_back_period_in_days': rec.get('lookBackPeriodInDays'),
                'last_refresh_timestamp': str(rec.get('lastRefreshTimestamp', '')),
                'current_performance_risk': rec.get('currentPerformanceRisk'),
            }

            options = rec.get('recommendationOptions', [])
            details['recommendation_options_count'] = len(options)

            resources.append({
                'service': 'compute-optimizer',
                'type': 'asg-recommendation',
                'id': asg_name,
                'arn': asg_arn,
                'name': asg_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    # Lambda Function Recommendations
    try:
        paginator = co.get_paginator('get_lambda_function_recommendations')
        for page in paginator.paginate():
            for rec in page.get('lambdaFunctionRecommendations', []):
                func_arn = rec.get('functionArn', '')
                func_version = rec.get('functionVersion', '$LATEST')

                details = {
                    'finding': rec.get('finding'),
                    'finding_reason_codes': rec.get('findingReasonCodes', []),
                    'current_memory_size': rec.get('currentMemorySize'),
                    'number_of_invocations': rec.get('numberOfInvocations'),
                    'look_back_period_in_days': rec.get('lookBackPeriodInDays'),
                    'last_refresh_timestamp': str(rec.get('lastRefreshTimestamp', '')),
                    'current_performance_risk': rec.get('currentPerformanceRisk'),
                }

                options = rec.get('memorySizeRecommendationOptions', [])
                details['recommendation_options_count'] = len(options)
                if options:
                    details['top_recommendation_memory_size'] = options[0].get('memorySize')

                resources.append({
                    'service': 'compute-optimizer',
                    'type': 'lambda-recommendation',
                    'id': f"{func_arn.split(':')[-1]}:{func_version}",
                    'arn': func_arn,
                    'name': func_arn.split(':')[-1] if func_arn else 'unknown',
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # EBS Volume Recommendations
    try:
        response = co.get_ebs_volume_recommendations()
        for rec in response.get('volumeRecommendations', []):
            volume_arn = rec.get('volumeArn', '')

            current_config = rec.get('currentConfiguration', {})
            details = {
                'finding': rec.get('finding'),
                'current_volume_type': current_config.get('volumeType'),
                'current_volume_size': current_config.get('volumeSize'),
                'current_iops': current_config.get('volumeBaselineIOPS'),
                'current_throughput': current_config.get('volumeBaselineThroughput'),
                'look_back_period_in_days': rec.get('lookBackPeriodInDays'),
                'last_refresh_timestamp': str(rec.get('lastRefreshTimestamp', '')),
                'current_performance_risk': rec.get('currentPerformanceRisk'),
            }

            options = rec.get('volumeRecommendationOptions', [])
            details['recommendation_options_count'] = len(options)

            resources.append({
                'service': 'compute-optimizer',
                'type': 'ebs-recommendation',
                'id': volume_arn.split('/')[-1] if volume_arn else 'unknown',
                'arn': volume_arn,
                'name': volume_arn.split('/')[-1] if volume_arn else 'unknown',
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    return resources



"""
EventBridge resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_events_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect EventBridge resources: event buses, rules, archives, connections, API destinations.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    events = session.client('events', region_name=region)

    # Event Buses (not pageable)
    event_buses = []
    try:
        response = events.list_event_buses()
        for bus in response.get('EventBuses', []):
            event_buses.append(bus)
            bus_name = bus['Name']
            bus_arn = bus['Arn']

            # Skip default bus (AWS-created)
            if bus_name == 'default':
                continue

            # Get tags
            tags = {}
            try:
                tag_response = events.list_tags_for_resource(ResourceARN=bus_arn)
                for tag in tag_response.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')
            except Exception:
                pass

            resources.append({
                'service': 'events',
                'type': 'event-bus',
                'id': bus_name,
                'arn': bus_arn,
                'name': bus_name,
                'region': region,
                'details': {
                    'policy': bus.get('Policy'),
                },
                'tags': tags
            })
    except Exception:
        pass

    # Rules (for each event bus)
    for bus in event_buses:
        bus_name = bus['Name']
        try:
            paginator = events.get_paginator('list_rules')
            for page in paginator.paginate(EventBusName=bus_name):
                for rule in page.get('Rules', []):
                    rule_name = rule['Name']
                    rule_arn = rule['Arn']

                    # Get tags
                    tags = {}
                    try:
                        tag_response = events.list_tags_for_resource(ResourceARN=rule_arn)
                        for tag in tag_response.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                    except Exception:
                        pass

                    # Get targets count
                    targets_count = 0
                    try:
                        targets_response = events.list_targets_by_rule(
                            Rule=rule_name,
                            EventBusName=bus_name
                        )
                        targets_count = len(targets_response.get('Targets', []))
                    except Exception:
                        pass

                    resources.append({
                        'service': 'events',
                        'type': 'rule',
                        'id': rule_name,
                        'arn': rule_arn,
                        'name': rule_name,
                        'region': region,
                        'details': {
                            'event_bus_name': bus_name,
                            'description': rule.get('Description'),
                            'state': rule.get('State'),
                            'schedule_expression': rule.get('ScheduleExpression'),
                            'event_pattern': rule.get('EventPattern'),
                            'managed_by': rule.get('ManagedBy'),
                            'targets_count': targets_count,
                        },
                        'tags': tags
                    })
        except Exception:
            pass

    # Archives
    try:
        response = events.list_archives()
        for archive in response.get('Archives', []):
            archive_name = archive['ArchiveName']
            archive_arn = f"arn:aws:events:{region}:{account_id}:archive/{archive_name}"

            resources.append({
                'service': 'events',
                'type': 'archive',
                'id': archive_name,
                'arn': archive_arn,
                'name': archive_name,
                'region': region,
                'details': {
                    'event_source_arn': archive.get('EventSourceArn'),
                    'state': archive.get('State'),
                    'state_reason': archive.get('StateReason'),
                    'retention_days': archive.get('RetentionDays'),
                    'size_bytes': archive.get('SizeBytes'),
                    'event_count': archive.get('EventCount'),
                    'creation_time': str(archive.get('CreationTime', '')),
                },
                'tags': {}
            })
    except Exception:
        pass

    # Connections (for API Destinations)
    try:
        response = events.list_connections()
        for conn in response.get('Connections', []):
            conn_name = conn['Name']
            conn_arn = conn['ConnectionArn']

            resources.append({
                'service': 'events',
                'type': 'connection',
                'id': conn_name,
                'arn': conn_arn,
                'name': conn_name,
                'region': region,
                'details': {
                    'state': conn.get('ConnectionState'),
                    'state_reason': conn.get('StateReason'),
                    'authorization_type': conn.get('AuthorizationType'),
                    'creation_time': str(conn.get('CreationTime', '')),
                    'last_modified_time': str(conn.get('LastModifiedTime', '')),
                    'last_authorized_time': str(conn.get('LastAuthorizedTime', '')),
                },
                'tags': {}
            })
    except Exception:
        pass

    # API Destinations
    try:
        response = events.list_api_destinations()
        for dest in response.get('ApiDestinations', []):
            dest_name = dest['Name']
            dest_arn = dest['ApiDestinationArn']

            resources.append({
                'service': 'events',
                'type': 'api-destination',
                'id': dest_name,
                'arn': dest_arn,
                'name': dest_name,
                'region': region,
                'details': {
                    'state': dest.get('ApiDestinationState'),
                    'connection_arn': dest.get('ConnectionArn'),
                    'invocation_endpoint': dest.get('InvocationEndpoint'),
                    'http_method': dest.get('HttpMethod'),
                    'invocation_rate_limit': dest.get('InvocationRateLimitPerSecond'),
                    'creation_time': str(dest.get('CreationTime', '')),
                    'last_modified_time': str(dest.get('LastModifiedTime', '')),
                },
                'tags': {}
            })
    except Exception:
        pass

    return resources



"""
CodeDeploy resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_codedeploy_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect CodeDeploy resources: applications, deployment groups, deployment configs.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    codedeploy = session.client('codedeploy', region_name=region)

    # Applications
    application_names = []
    try:
        paginator = codedeploy.get_paginator('list_applications')
        for page in paginator.paginate():
            application_names.extend(page.get('applications', []))
    except Exception:
        pass

    for app_name in application_names:
        try:
            app_response = codedeploy.get_application(applicationName=app_name)
            app = app_response.get('application', {})

            app_arn = f"arn:aws:codedeploy:{region}:{account_id}:application:{app_name}"

            # Get tags
            tags = {}
            try:
                tag_response = codedeploy.list_tags_for_resource(ResourceArn=app_arn)
                for tag in tag_response.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')
            except Exception:
                pass

            resources.append({
                'service': 'codedeploy',
                'type': 'application',
                'id': app_name,
                'arn': app_arn,
                'name': app_name,
                'region': region,
                'details': {
                    'compute_platform': app.get('computePlatform'),
                    'linked_to_github': app.get('linkedToGitHub'),
                    'github_account_name': app.get('gitHubAccountName'),
                    'create_time': str(app.get('createTime', '')),
                },
                'tags': tags
            })

            # Deployment Groups for this application
            try:
                dg_paginator = codedeploy.get_paginator('list_deployment_groups')
                dg_names = []
                for dg_page in dg_paginator.paginate(applicationName=app_name):
                    dg_names.extend(dg_page.get('deploymentGroups', []))

                # Batch get deployment group details
                if dg_names:
                    dg_response = codedeploy.batch_get_deployment_groups(
                        applicationName=app_name,
                        deploymentGroupNames=dg_names
                    )

                    for dg in dg_response.get('deploymentGroupsInfo', []):
                        dg_name = dg['deploymentGroupName']
                        dg_id = dg['deploymentGroupId']
                        dg_arn = f"arn:aws:codedeploy:{region}:{account_id}:deploymentgroup:{app_name}/{dg_name}"

                        # Get tags
                        dg_tags = {}
                        try:
                            dg_tag_response = codedeploy.list_tags_for_resource(ResourceArn=dg_arn)
                            for tag in dg_tag_response.get('Tags', []):
                                dg_tags[tag.get('Key', '')] = tag.get('Value', '')
                        except Exception:
                            pass

                        resources.append({
                            'service': 'codedeploy',
                            'type': 'deployment-group',
                            'id': dg_id,
                            'arn': dg_arn,
                            'name': dg_name,
                            'region': region,
                            'details': {
                                'application_name': app_name,
                                'deployment_config_name': dg.get('deploymentConfigName'),
                                'service_role_arn': dg.get('serviceRoleArn'),
                                'deployment_style': dg.get('deploymentStyle'),
                                'blue_green_deployment_config': bool(dg.get('blueGreenDeploymentConfiguration')),
                                'load_balancer_info': bool(dg.get('loadBalancerInfo')),
                                'ec2_tag_set': bool(dg.get('ec2TagSet')),
                                'ecs_services': len(dg.get('ecsServices', [])),
                                'auto_scaling_groups': len(dg.get('autoScalingGroups', [])),
                                'compute_platform': dg.get('computePlatform'),
                            },
                            'tags': dg_tags
                        })
            except Exception:
                pass
        except Exception:
            pass

    # Custom Deployment Configs (not predefined)
    try:
        paginator = codedeploy.get_paginator('list_deployment_configs')
        for page in paginator.paginate():
            for config_name in page.get('deploymentConfigsList', []):
                # Skip predefined configs
                if config_name.startswith('CodeDeployDefault.'):
                    continue

                try:
                    config_response = codedeploy.get_deployment_config(
                        deploymentConfigName=config_name
                    )
                    config = config_response.get('deploymentConfigInfo', {})

                    config_id = config.get('deploymentConfigId')

                    resources.append({
                        'service': 'codedeploy',
                        'type': 'deployment-config',
                        'id': config_id or config_name,
                        'arn': f"arn:aws:codedeploy:{region}:{account_id}:deploymentconfig:{config_name}",
                        'name': config_name,
                        'region': region,
                        'details': {
                            'compute_platform': config.get('computePlatform'),
                            'minimum_healthy_hosts': config.get('minimumHealthyHosts'),
                            'traffic_routing_config': config.get('trafficRoutingConfig'),
                            'create_time': str(config.get('createTime', '')),
                        },
                        'tags': {}
                    })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
AWS Cloud Map (Service Discovery) resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_servicediscovery_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Cloud Map resources: namespaces, services, and instances.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    sd = session.client('servicediscovery', region_name=region)

    # Namespaces
    try:
        paginator = sd.get_paginator('list_namespaces')
        for page in paginator.paginate():
            for ns in page.get('Namespaces', []):
                ns_id = ns['Id']
                ns_arn = ns['Arn']
                ns_name = ns.get('Name', ns_id)

                details = {
                    'type': ns.get('Type'),
                    'description': ns.get('Description'),
                    'service_count': ns.get('ServiceCount'),
                    'create_date': str(ns.get('CreateDate', '')),
                }

                # Get additional namespace details
                try:
                    ns_detail = sd.get_namespace(Id=ns_id)
                    ns_info = ns_detail.get('Namespace', {})
                    props = ns_info.get('Properties', {})

                    dns_props = props.get('DnsProperties', {})
                    if dns_props:
                        details['hosted_zone_id'] = dns_props.get('HostedZoneId')
                        details['soa_ttl'] = dns_props.get('SOA', {}).get('TTL')

                    http_props = props.get('HttpProperties', {})
                    if http_props:
                        details['http_name'] = http_props.get('HttpName')
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = sd.list_tags_for_resource(ResourceARN=ns_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'servicediscovery',
                    'type': 'namespace',
                    'id': ns_id,
                    'arn': ns_arn,
                    'name': ns_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })

                # Services in this namespace
                try:
                    svc_paginator = sd.get_paginator('list_services')
                    for svc_page in svc_paginator.paginate(
                        Filters=[{'Name': 'NAMESPACE_ID', 'Values': [ns_id], 'Condition': 'EQ'}]
                    ):
                        for svc in svc_page.get('Services', []):
                            svc_id = svc['Id']
                            svc_arn = svc['Arn']
                            svc_name = svc.get('Name', svc_id)

                            svc_details = {
                                'namespace_id': ns_id,
                                'namespace_name': ns_name,
                                'description': svc.get('Description'),
                                'instance_count': svc.get('InstanceCount'),
                                'create_date': str(svc.get('CreateDate', '')),
                                'type': svc.get('Type'),
                            }

                            # DNS config
                            dns_config = svc.get('DnsConfig', {})
                            if dns_config:
                                svc_details['routing_policy'] = dns_config.get('RoutingPolicy')
                                svc_details['dns_records'] = dns_config.get('DnsRecords', [])

                            # Health check config
                            hc_config = svc.get('HealthCheckConfig', {})
                            if hc_config:
                                svc_details['health_check_type'] = hc_config.get('Type')
                                svc_details['health_check_path'] = hc_config.get('ResourcePath')
                                svc_details['health_check_failure_threshold'] = hc_config.get('FailureThreshold')

                            hc_custom = svc.get('HealthCheckCustomConfig', {})
                            if hc_custom:
                                svc_details['custom_health_check_failure_threshold'] = hc_custom.get('FailureThreshold')

                            # Get service tags
                            svc_tags = {}
                            try:
                                svc_tag_response = sd.list_tags_for_resource(ResourceARN=svc_arn)
                                for tag in svc_tag_response.get('Tags', []):
                                    svc_tags[tag.get('Key', '')] = tag.get('Value', '')
                            except Exception:
                                pass

                            resources.append({
                                'service': 'servicediscovery',
                                'type': 'service',
                                'id': svc_id,
                                'arn': svc_arn,
                                'name': svc_name,
                                'region': region,
                                'details': svc_details,
                                'tags': svc_tags
                            })

                            # Instances for this service
                            try:
                                inst_paginator = sd.get_paginator('list_instances')
                                for inst_page in inst_paginator.paginate(ServiceId=svc_id):
                                    for inst in inst_page.get('Instances', []):
                                        inst_id = inst['Id']
                                        inst_attrs = inst.get('Attributes', {})

                                        inst_details = {
                                            'service_id': svc_id,
                                            'service_name': svc_name,
                                            'namespace_id': ns_id,
                                            'namespace_name': ns_name,
                                            'aws_instance_ipv4': inst_attrs.get('AWS_INSTANCE_IPV4'),
                                            'aws_instance_ipv6': inst_attrs.get('AWS_INSTANCE_IPV6'),
                                            'aws_instance_port': inst_attrs.get('AWS_INSTANCE_PORT'),
                                            'aws_alias_dns_name': inst_attrs.get('AWS_ALIAS_DNS_NAME'),
                                            'aws_init_health_status': inst_attrs.get('AWS_INIT_HEALTH_STATUS'),
                                        }

                                        # Add any custom attributes
                                        custom_attrs = {k: v for k, v in inst_attrs.items()
                                                       if not k.startswith('AWS_')}
                                        if custom_attrs:
                                            inst_details['custom_attributes'] = custom_attrs

                                        resources.append({
                                            'service': 'servicediscovery',
                                            'type': 'instance',
                                            'id': inst_id,
                                            'arn': f"arn:aws:servicediscovery:{region}:{account_id}:service/{svc_id}/instance/{inst_id}",
                                            'name': inst_id,
                                            'region': region,
                                            'details': inst_details,
                                            'tags': {}
                                        })
                            except Exception:
                                pass
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
AWS Amazon Detective resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_detective_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Amazon Detective resources: behavior graphs, members, and investigations.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    detective = session.client('detective', region_name=region)

    # Behavior Graphs
    try:
        paginator = detective.get_paginator('list_graphs')
        for page in paginator.paginate():
            for graph in page.get('GraphList', []):
                graph_arn = graph['Arn']
                graph_id = graph_arn.split('/')[-1]

                details = {
                    'created_time': str(graph.get('CreatedTime', '')),
                }

                # Get tags
                tags = {}
                try:
                    tag_response = detective.list_tags_for_resource(ResourceArn=graph_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'detective',
                    'type': 'graph',
                    'id': graph_id,
                    'arn': graph_arn,
                    'name': graph_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })

                # Members in this graph
                try:
                    member_paginator = detective.get_paginator('list_members')
                    for member_page in member_paginator.paginate(GraphArn=graph_arn):
                        for member in member_page.get('MemberDetails', []):
                            member_id = member['AccountId']

                            member_details = {
                                'graph_arn': graph_arn,
                                'email_address': member.get('EmailAddress'),
                                'status': member.get('Status'),
                                'disabled_reason': member.get('DisabledReason'),
                                'invited_time': str(member.get('InvitedTime', '')),
                                'updated_time': str(member.get('UpdatedTime', '')),
                                'volume_usage_in_bytes': member.get('VolumeUsageInBytes'),
                                'volume_usage_updated_time': str(member.get('VolumeUsageUpdatedTime', '')),
                                'percent_of_graph_utilization': member.get('PercentOfGraphUtilization'),
                                'invitation_type': member.get('InvitationType'),
                            }

                            # Get datasource packages for this member
                            try:
                                ds_response = detective.batch_get_graph_member_datasources(
                                    GraphArn=graph_arn,
                                    AccountIds=[member_id]
                                )
                                for ds_member in ds_response.get('MemberDatasources', []):
                                    if ds_member.get('AccountId') == member_id:
                                        member_details['datasource_packages'] = list(ds_member.get('DatasourcePackageIngestHistory', {}).keys())
                            except Exception:
                                pass

                            resources.append({
                                'service': 'detective',
                                'type': 'member',
                                'id': member_id,
                                'arn': f"{graph_arn}/member/{member_id}",
                                'name': member_id,
                                'region': region,
                                'details': member_details,
                                'tags': {}
                            })
                except Exception:
                    pass

                # Investigations in this graph
                try:
                    inv_paginator = detective.get_paginator('list_investigations')
                    for inv_page in inv_paginator.paginate(GraphArn=graph_arn):
                        for investigation in inv_page.get('InvestigationDetails', []):
                            inv_id = investigation['InvestigationId']

                            inv_details = {
                                'graph_arn': graph_arn,
                                'entity_arn': investigation.get('EntityArn'),
                                'entity_type': investigation.get('EntityType'),
                                'severity': investigation.get('Severity'),
                                'status': investigation.get('Status'),
                                'state': investigation.get('State'),
                                'created_time': str(investigation.get('CreatedTime', '')),
                            }

                            resources.append({
                                'service': 'detective',
                                'type': 'investigation',
                                'id': inv_id,
                                'arn': f"{graph_arn}/investigation/{inv_id}",
                                'name': inv_id,
                                'region': region,
                                'details': inv_details,
                                'tags': {}
                            })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
Classic ELB resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_elb_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Classic ELB resources.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    elb = session.client('elb', region_name=region)

    try:
        paginator = elb.get_paginator('describe_load_balancers')
        for page in paginator.paginate():
            for lb in page.get('LoadBalancerDescriptions', []):
                lb_name = lb['LoadBalancerName']

                # Get tags
                tags = {}
                try:
                    tag_response = elb.describe_tags(LoadBalancerNames=[lb_name])
                    for tag_desc in tag_response.get('TagDescriptions', []):
                        for tag in tag_desc.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'elb',
                    'type': 'classic-load-balancer',
                    'id': lb_name,
                    'arn': f"arn:aws:elasticloadbalancing:{region}:{account_id}:loadbalancer/{lb_name}",
                    'name': lb_name,
                    'region': region,
                    'details': {
                        'dns_name': lb.get('DNSName'),
                        'scheme': lb.get('Scheme'),
                        'vpc_id': lb.get('VPCId'),
                        'availability_zones': lb.get('AvailabilityZones', []),
                        'subnets': lb.get('Subnets', []),
                        'security_groups': lb.get('SecurityGroups', []),
                        'instances': [i.get('InstanceId') for i in lb.get('Instances', [])],
                        'listeners': [
                            {
                                'protocol': l.get('Protocol'),
                                'lb_port': l.get('LoadBalancerPort'),
                                'instance_protocol': l.get('InstanceProtocol'),
                                'instance_port': l.get('InstancePort'),
                            }
                            for l in lb.get('ListenerDescriptions', [])
                            for l in [l.get('Listener', {})]
                        ],
                        'created_time': str(lb.get('CreatedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS Location Service resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# Location Service supported regions (from https://docs.aws.amazon.com/location/latest/developerguide/location-regions.html)
LOCATION_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-2',
    'ap-south-1', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1',
    'ca-central-1',
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-north-1',
    'sa-east-1',
}


def collect_location_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Location Service resources: maps, place indexes, route calculators,
    geofence collections, and trackers.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in LOCATION_REGIONS:
        return []

    resources = []
    location = session.client('location', region_name=region)

    # Maps
    try:
        paginator = location.get_paginator('list_maps')
        for page in paginator.paginate():
            for map_entry in page.get('Entries', []):
                map_name = map_entry['MapName']
                map_arn = f"arn:aws:geo:{region}:{account_id}:map/{map_name}"

                details = {
                    'description': map_entry.get('Description'),
                    'data_source': map_entry.get('DataSource'),
                    'pricing_plan': map_entry.get('PricingPlan'),
                }

                resources.append({
                    'service': 'location',
                    'type': 'map',
                    'id': map_name,
                    'arn': map_arn,
                    'name': map_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Place Indexes
    try:
        paginator = location.get_paginator('list_place_indexes')
        for page in paginator.paginate():
            for index in page.get('Entries', []):
                index_name = index['IndexName']
                index_arn = f"arn:aws:geo:{region}:{account_id}:place-index/{index_name}"

                details = {
                    'description': index.get('Description'),
                    'data_source': index.get('DataSource'),
                    'pricing_plan': index.get('PricingPlan'),
                }

                resources.append({
                    'service': 'location',
                    'type': 'place-index',
                    'id': index_name,
                    'arn': index_arn,
                    'name': index_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Route Calculators
    try:
        paginator = location.get_paginator('list_route_calculators')
        for page in paginator.paginate():
            for calc in page.get('Entries', []):
                calc_name = calc['CalculatorName']
                calc_arn = f"arn:aws:geo:{region}:{account_id}:route-calculator/{calc_name}"

                details = {
                    'description': calc.get('Description'),
                    'data_source': calc.get('DataSource'),
                    'pricing_plan': calc.get('PricingPlan'),
                }

                resources.append({
                    'service': 'location',
                    'type': 'route-calculator',
                    'id': calc_name,
                    'arn': calc_arn,
                    'name': calc_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Geofence Collections
    try:
        paginator = location.get_paginator('list_geofence_collections')
        for page in paginator.paginate():
            for collection in page.get('Entries', []):
                coll_name = collection['CollectionName']
                coll_arn = f"arn:aws:geo:{region}:{account_id}:geofence-collection/{coll_name}"

                details = {
                    'description': collection.get('Description'),
                    'pricing_plan': collection.get('PricingPlan'),
                }

                resources.append({
                    'service': 'location',
                    'type': 'geofence-collection',
                    'id': coll_name,
                    'arn': coll_arn,
                    'name': coll_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Trackers
    try:
        paginator = location.get_paginator('list_trackers')
        for page in paginator.paginate():
            for tracker in page.get('Entries', []):
                tracker_name = tracker['TrackerName']
                tracker_arn = f"arn:aws:geo:{region}:{account_id}:tracker/{tracker_name}"

                details = {
                    'description': tracker.get('Description'),
                    'pricing_plan': tracker.get('PricingPlan'),
                }

                resources.append({
                    'service': 'location',
                    'type': 'tracker',
                    'id': tracker_name,
                    'arn': tracker_arn,
                    'name': tracker_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Resilience Hub resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_resiliencehub_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Resilience Hub resources: apps and resiliency policies.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    resiliencehub = session.client('resiliencehub', region_name=region)

    # Apps
    try:
        paginator = resiliencehub.get_paginator('list_apps')
        for page in paginator.paginate():
            for app in page.get('appSummaries', []):
                app_arn = app['appArn']
                app_name = app.get('name', app_arn.split('/')[-1])

                details = {
                    'description': app.get('description'),
                    'status': app.get('status'),
                    'compliance_status': app.get('complianceStatus'),
                    'resiliency_score': app.get('resiliencyScore'),
                    'assessment_schedule': app.get('assessmentSchedule'),
                }

                resources.append({
                    'service': 'resiliencehub',
                    'type': 'app',
                    'id': app_arn.split('/')[-1],
                    'arn': app_arn,
                    'name': app_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Resiliency Policies
    try:
        paginator = resiliencehub.get_paginator('list_resiliency_policies')
        for page in paginator.paginate():
            for policy in page.get('resiliencyPolicies', []):
                policy_arn = policy['policyArn']
                policy_name = policy.get('policyName', policy_arn.split('/')[-1])

                details = {
                    'description': policy.get('policyDescription'),
                    'tier': policy.get('tier'),
                    'data_location_constraint': policy.get('dataLocationConstraint'),
                }

                resources.append({
                    'service': 'resiliencehub',
                    'type': 'resiliency-policy',
                    'id': policy_arn.split('/')[-1],
                    'arn': policy_arn,
                    'name': policy_name,
                    'region': region,
                    'details': details,
                    'tags': policy.get('tags', {})
                })
    except Exception:
        pass

    return resources



"""
Inspector2 resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_inspector2_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Inspector2 resources: coverage statistics, filters.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    inspector2 = session.client('inspector2', region_name=region)

    # Get Inspector2 status
    try:
        status_response = inspector2.batch_get_account_status(
            accountIds=[account_id]
        )

        for account_status in status_response.get('accounts', []):
            if account_status.get('state', {}).get('status') == 'ENABLED':
                resources.append({
                    'service': 'inspector2',
                    'type': 'account-status',
                    'id': account_id,
                    'arn': f"arn:aws:inspector2:{region}:{account_id}:account",
                    'name': f"inspector2-{account_id}",
                    'region': region,
                    'details': {
                        'status': account_status.get('state', {}).get('status'),
                        'resource_status': account_status.get('resourceState'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Filters
    try:
        paginator = inspector2.get_paginator('list_filters')
        for page in paginator.paginate():
            for filter_item in page.get('filters', []):
                filter_arn = filter_item['arn']
                filter_name = filter_item['name']

                # Get tags
                tags = filter_item.get('tags', {})

                resources.append({
                    'service': 'inspector2',
                    'type': 'filter',
                    'id': filter_name,
                    'arn': filter_arn,
                    'name': filter_name,
                    'region': region,
                    'details': {
                        'description': filter_item.get('description'),
                        'action': filter_item.get('action'),
                        'owner_id': filter_item.get('ownerId'),
                        'reason': filter_item.get('reason'),
                        'created_at': str(filter_item.get('createdAt', '')),
                        'updated_at': str(filter_item.get('updatedAt', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources
